"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from typing import overload

from zospy.api._ZOSAPI.Tools import ISystemTool
from zospy.api._ZOSAPI.Tools.General import RayPatternType

__all__ = (
    "BackgroundOptions",
    "BrightnessOptions",
    "CameraViewpointOptions",
    "ColorRaysByCrossSectionOptions",
    "ColorRaysByNSCOptions",
    "ColorRaysByOptions",
    "DetectorDisplayModeOptions",
    "DetectorPixelColorOptions",
    "DrawSectionOptions",
    "I3DViewerExport",
    "ICrossSectionExport",
    "IImageExportData",
    "IImagePixel",
    "INSC3DLayoutExport",
    "INSCShadedModelExport",
    "IShadedModelExport",
    "LineThicknessOptions",
    "NumberSegmentsOptions",
    "OpacityOptions",
    "RayTraceOptions",
    "RealPupilOptions",
)

class BackgroundOptions:
    White = 0
    Black = 1
    Red = 2
    Green = 3
    Blue = 4
    DarkGreen = 5
    DarkBlue = 6
    Color01 = 7
    Color02 = 8
    Color03 = 9
    Color04 = 10
    Color05 = 11
    Color06 = 12
    Color07 = 13
    Color08 = 14
    Color09 = 15
    Color10 = 16
    Color11 = 17
    Color12 = 18
    Color13 = 19
    Color14 = 20
    Color15 = 21
    Color16 = 22
    Color17 = 23
    Color18 = 24
    Color19 = 25
    Color20 = 26
    Color21 = 27
    Color22 = 28
    Color23 = 29
    Color24 = 30
    Gradient01 = 31
    Gradient02 = 32
    Gradient03 = 33
    Gradient04 = 34
    Gradient05 = 35
    Gradient06 = 36
    Gradient07 = 37
    Gradient08 = 38
    Gradient09 = 39
    Gradient10 = 40
    GradientUser = 41

class BrightnessOptions:
    P100 = 0
    P90 = 1
    P80 = 2
    P70 = 3
    P60 = 4
    P50 = 5
    P40 = 6
    P30 = 7
    P20 = 8
    P10 = 9

class CameraViewpointOptions:
    Isometric = 0
    XY = 1
    YZ = 2
    XZ = 3

class ColorRaysByCrossSectionOptions:
    Fields = 0
    Waves = 1
    Wavelength = 2

class ColorRaysByNSCOptions:
    SourceNumber = 0
    WaveNumber = 1
    ConfigNumber = 2
    Wavelength = 3
    SegmentNumber = 4

class ColorRaysByOptions:
    Fields = 0
    Waves = 1
    Config = 2
    Wavelength = 3

class DetectorDisplayModeOptions:
    Consider = 0
    GreyScaleFlux = 1
    InverseGreyScaleFlux = 2
    FalseColorFlux = 3
    InverseFalseColorFlux = 4
    GreyScaleIrradiance = 5
    InverseGreyScaleIrradiance = 6
    FalseColorIrradiance = 7
    InverseFalseColorIrradiance = 8

class DetectorPixelColorOptions:
    DoNotColorIndividualPoints = 0
    ByRaysOnLayout = 1
    ByLastAnalysis = 2

class DrawSectionOptions:
    P100 = 0
    P75 = 1
    P50 = 2
    P25 = 3

class I3DViewerExport(ISystemTool):
    @property
    def SaveImageAsFile(self) -> bool: ...
    @SaveImageAsFile.setter
    def SaveImageAsFile(self, value: bool) -> None: ...
    @property
    def OutputFileName(self) -> str: ...
    @OutputFileName.setter
    def OutputFileName(self, value: str) -> None: ...
    @property
    def IsValidFileName(self) -> bool: ...
    @property
    def OutputPixelWidth(self) -> int: ...
    @OutputPixelWidth.setter
    def OutputPixelWidth(self, value: int) -> None: ...
    @property
    def OutputPixelHeight(self) -> int: ...
    @OutputPixelHeight.setter
    def OutputPixelHeight(self, value: int) -> None: ...
    @property
    def StartSurface(self) -> int: ...
    @StartSurface.setter
    def StartSurface(self, value: int) -> None: ...
    @property
    def EndSurface(self) -> int: ...
    @EndSurface.setter
    def EndSurface(self, value: int) -> None: ...
    @property
    def NumberOfRays(self) -> int: ...
    @NumberOfRays.setter
    def NumberOfRays(self, value: int) -> None: ...
    @property
    def Wavelength(self) -> int: ...
    @Wavelength.setter
    def Wavelength(self, value: int) -> None: ...
    @property
    def Field(self) -> int: ...
    @Field.setter
    def Field(self, value: int) -> None: ...
    @property
    def RayPattern(self) -> RayPatternType: ...
    @RayPattern.setter
    def RayPattern(self, value: RayPatternType) -> None: ...
    @property
    def ColorRaysBy(self) -> ColorRaysByOptions: ...
    @ColorRaysBy.setter
    def ColorRaysBy(self, value: ColorRaysByOptions) -> None: ...
    @property
    def DeleteVignetted(self) -> bool: ...
    @DeleteVignetted.setter
    def DeleteVignetted(self, value: bool) -> None: ...
    @property
    def HideLensFaces(self) -> bool: ...
    @HideLensFaces.setter
    def HideLensFaces(self, value: bool) -> None: ...
    @property
    def HideLensEdges(self) -> bool: ...
    @HideLensEdges.setter
    def HideLensEdges(self, value: bool) -> None: ...
    @property
    def HideXBars(self) -> bool: ...
    @HideXBars.setter
    def HideXBars(self, value: bool) -> None: ...
    @property
    def DrawParaxialPupils(self) -> bool: ...
    @DrawParaxialPupils.setter
    def DrawParaxialPupils(self, value: bool) -> None: ...
    @property
    def FletchRays(self) -> bool: ...
    @FletchRays.setter
    def FletchRays(self, value: bool) -> None: ...
    @property
    def SplitNSCRays(self) -> bool: ...
    @SplitNSCRays.setter
    def SplitNSCRays(self, value: bool) -> None: ...
    @property
    def ScatterNSCRays(self) -> bool: ...
    @ScatterNSCRays.setter
    def ScatterNSCRays(self, value: bool) -> None: ...
    @property
    def DrawRealEntrancePupils(self) -> RealPupilOptions: ...
    @DrawRealEntrancePupils.setter
    def DrawRealEntrancePupils(self, value: RealPupilOptions) -> None: ...
    @property
    def DrawRealExitPupils(self) -> RealPupilOptions: ...
    @DrawRealExitPupils.setter
    def DrawRealExitPupils(self, value: RealPupilOptions) -> None: ...
    @property
    def SurfaceLineThickness(self) -> LineThicknessOptions: ...
    @SurfaceLineThickness.setter
    def SurfaceLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def RaysLineThickness(self) -> LineThicknessOptions: ...
    @RaysLineThickness.setter
    def RaysLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def ConfigurationAll(self) -> bool: ...
    @ConfigurationAll.setter
    def ConfigurationAll(self, value: bool) -> None: ...
    @property
    def ConfigurationCurrent(self) -> bool: ...
    @ConfigurationCurrent.setter
    def ConfigurationCurrent(self, value: bool) -> None: ...
    @property
    def ConfigurationOffsetX(self) -> float: ...
    @ConfigurationOffsetX.setter
    def ConfigurationOffsetX(self, value: float) -> None: ...
    @property
    def ConfigurationOffsetY(self) -> float: ...
    @ConfigurationOffsetY.setter
    def ConfigurationOffsetY(self, value: float) -> None: ...
    @property
    def ConfigurationOffsetZ(self) -> float: ...
    @ConfigurationOffsetZ.setter
    def ConfigurationOffsetZ(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleX(self) -> float: ...
    @CameraViewpointAngleX.setter
    def CameraViewpointAngleX(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleY(self) -> float: ...
    @CameraViewpointAngleY.setter
    def CameraViewpointAngleY(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleZ(self) -> float: ...
    @CameraViewpointAngleZ.setter
    def CameraViewpointAngleZ(self, value: float) -> None: ...
    @property
    def ImageExportData(self) -> IImageExportData: ...
    def SetCameraViewpoint(self, cameraViewpoint: CameraViewpointOptions) -> None: ...
    def SetConfigurationEnabled(self, config: int, enabled: bool) -> None: ...

class ICrossSectionExport(ISystemTool):
    @property
    def SaveImageAsFile(self) -> bool: ...
    @SaveImageAsFile.setter
    def SaveImageAsFile(self, value: bool) -> None: ...
    @property
    def OutputFileName(self) -> str: ...
    @OutputFileName.setter
    def OutputFileName(self, value: str) -> None: ...
    @property
    def IsValidFileName(self) -> bool: ...
    @property
    def OutputPixelWidth(self) -> int: ...
    @OutputPixelWidth.setter
    def OutputPixelWidth(self, value: int) -> None: ...
    @property
    def OutputPixelHeight(self) -> int: ...
    @OutputPixelHeight.setter
    def OutputPixelHeight(self, value: int) -> None: ...
    @property
    def StartSurface(self) -> int: ...
    @StartSurface.setter
    def StartSurface(self, value: int) -> None: ...
    @property
    def EndSurface(self) -> int: ...
    @EndSurface.setter
    def EndSurface(self, value: int) -> None: ...
    @property
    def NumberOfRays(self) -> int: ...
    @NumberOfRays.setter
    def NumberOfRays(self, value: int) -> None: ...
    @property
    def YStretch(self) -> float: ...
    @YStretch.setter
    def YStretch(self, value: float) -> None: ...
    @property
    def Wavelength(self) -> int: ...
    @Wavelength.setter
    def Wavelength(self, value: int) -> None: ...
    @property
    def Field(self) -> int: ...
    @Field.setter
    def Field(self, value: int) -> None: ...
    @property
    def ColorRaysBy(self) -> ColorRaysByCrossSectionOptions: ...
    @ColorRaysBy.setter
    def ColorRaysBy(self, value: ColorRaysByCrossSectionOptions) -> None: ...
    @property
    def UpperPupil(self) -> float: ...
    @UpperPupil.setter
    def UpperPupil(self, value: float) -> None: ...
    @property
    def LowerPupil(self) -> float: ...
    @LowerPupil.setter
    def LowerPupil(self, value: float) -> None: ...
    @property
    def FletchRays(self) -> bool: ...
    @FletchRays.setter
    def FletchRays(self, value: bool) -> None: ...
    @property
    def DeleteVignetted(self) -> bool: ...
    @DeleteVignetted.setter
    def DeleteVignetted(self, value: bool) -> None: ...
    @property
    def MarginalAndChiefRayOnly(self) -> bool: ...
    @MarginalAndChiefRayOnly.setter
    def MarginalAndChiefRayOnly(self, value: bool) -> None: ...
    @property
    def SurfaceLineThickness(self) -> LineThicknessOptions: ...
    @SurfaceLineThickness.setter
    def SurfaceLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def RaysLineThickness(self) -> LineThicknessOptions: ...
    @RaysLineThickness.setter
    def RaysLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def Configuration(self) -> int: ...
    @Configuration.setter
    def Configuration(self, value: int) -> None: ...
    @property
    def ImageExportData(self) -> IImageExportData: ...
    def SetFieldsAll(self) -> None: ...
    def SetWavelengthsAll(self) -> None: ...

class IImageExportData:
    @property
    def Width(self) -> int: ...
    @property
    def Height(self) -> int: ...
    @property
    def Values(self) -> list[list[IImagePixel]]: ...
    def FillValues(self, fullSize: int) -> tuple[list[int], list[int], list[int]]: ...
    @overload
    def GetImagePixel(self, index: int) -> IImagePixel: ...
    @overload
    def GetImagePixel(self, x: int, y: int) -> IImagePixel: ...

class IImagePixel:
    @property
    def R(self) -> int: ...
    @property
    def G(self) -> int: ...
    @property
    def B(self) -> int: ...

class INSC3DLayoutExport(ISystemTool):
    @property
    def SaveImageAsFile(self) -> bool: ...
    @SaveImageAsFile.setter
    def SaveImageAsFile(self, value: bool) -> None: ...
    @property
    def OutputFileName(self) -> str: ...
    @OutputFileName.setter
    def OutputFileName(self, value: str) -> None: ...
    @property
    def IsValidFileName(self) -> bool: ...
    @property
    def OutputPixelWidth(self) -> int: ...
    @OutputPixelWidth.setter
    def OutputPixelWidth(self, value: int) -> None: ...
    @property
    def OutputPixelHeight(self) -> int: ...
    @OutputPixelHeight.setter
    def OutputPixelHeight(self, value: int) -> None: ...
    @property
    def RayDatabase(self) -> str: ...
    @RayDatabase.setter
    def RayDatabase(self, value: str) -> None: ...
    @property
    def Filter(self) -> str: ...
    @Filter.setter
    def Filter(self, value: str) -> None: ...
    @property
    def RayTrace(self) -> RayTraceOptions: ...
    @RayTrace.setter
    def RayTrace(self, value: RayTraceOptions) -> None: ...
    @property
    def ColorRaysBy(self) -> ColorRaysByNSCOptions: ...
    @ColorRaysBy.setter
    def ColorRaysBy(self, value: ColorRaysByNSCOptions) -> None: ...
    @property
    def UsePolarization(self) -> bool: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...
    @property
    def FletchRays(self) -> bool: ...
    @FletchRays.setter
    def FletchRays(self, value: bool) -> None: ...
    @property
    def SplitNSCRays(self) -> bool: ...
    @SplitNSCRays.setter
    def SplitNSCRays(self, value: bool) -> None: ...
    @property
    def ScatterNSCRays(self) -> bool: ...
    @ScatterNSCRays.setter
    def ScatterNSCRays(self, value: bool) -> None: ...
    @property
    def SurfaceLineThickness(self) -> LineThicknessOptions: ...
    @SurfaceLineThickness.setter
    def SurfaceLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def RaysLineThickness(self) -> LineThicknessOptions: ...
    @RaysLineThickness.setter
    def RaysLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def ConfigurationAll(self) -> bool: ...
    @ConfigurationAll.setter
    def ConfigurationAll(self, value: bool) -> None: ...
    @property
    def ConfigurationCurrent(self) -> bool: ...
    @ConfigurationCurrent.setter
    def ConfigurationCurrent(self, value: bool) -> None: ...
    @property
    def ConfigurationOffsetX(self) -> float: ...
    @ConfigurationOffsetX.setter
    def ConfigurationOffsetX(self, value: float) -> None: ...
    @property
    def ConfigurationOffsetY(self) -> float: ...
    @ConfigurationOffsetY.setter
    def ConfigurationOffsetY(self, value: float) -> None: ...
    @property
    def ConfigurationOffsetZ(self) -> float: ...
    @ConfigurationOffsetZ.setter
    def ConfigurationOffsetZ(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleX(self) -> float: ...
    @CameraViewpointAngleX.setter
    def CameraViewpointAngleX(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleY(self) -> float: ...
    @CameraViewpointAngleY.setter
    def CameraViewpointAngleY(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleZ(self) -> float: ...
    @CameraViewpointAngleZ.setter
    def CameraViewpointAngleZ(self, value: float) -> None: ...
    @property
    def ImageExportData(self) -> IImageExportData: ...
    def SetCameraViewpoint(self, cameraViewpoint: CameraViewpointOptions) -> None: ...
    def SetConfigurationEnabled(self, config: int, enabled: bool) -> None: ...

class INSCShadedModelExport(ISystemTool):
    @property
    def SaveImageAsFile(self) -> bool: ...
    @SaveImageAsFile.setter
    def SaveImageAsFile(self, value: bool) -> None: ...
    @property
    def OutputFileName(self) -> str: ...
    @OutputFileName.setter
    def OutputFileName(self, value: str) -> None: ...
    @property
    def IsValidFileName(self) -> bool: ...
    @property
    def OutputPixelWidth(self) -> int: ...
    @OutputPixelWidth.setter
    def OutputPixelWidth(self, value: int) -> None: ...
    @property
    def OutputPixelHeight(self) -> int: ...
    @OutputPixelHeight.setter
    def OutputPixelHeight(self, value: int) -> None: ...
    @property
    def RayDatabase(self) -> str: ...
    @RayDatabase.setter
    def RayDatabase(self, value: str) -> None: ...
    @property
    def Filter(self) -> str: ...
    @Filter.setter
    def Filter(self, value: str) -> None: ...
    @property
    def RayTrace(self) -> RayTraceOptions: ...
    @RayTrace.setter
    def RayTrace(self, value: RayTraceOptions) -> None: ...
    @property
    def ColorRaysBy(self) -> ColorRaysByNSCOptions: ...
    @ColorRaysBy.setter
    def ColorRaysBy(self, value: ColorRaysByNSCOptions) -> None: ...
    @property
    def DetectorPixelColorMode(self) -> DetectorPixelColorOptions: ...
    @DetectorPixelColorMode.setter
    def DetectorPixelColorMode(self, value: DetectorPixelColorOptions) -> None: ...
    @property
    def UsePolarization(self) -> bool: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...
    @property
    def FletchRays(self) -> bool: ...
    @FletchRays.setter
    def FletchRays(self, value: bool) -> None: ...
    @property
    def SplitNSCRays(self) -> bool: ...
    @SplitNSCRays.setter
    def SplitNSCRays(self, value: bool) -> None: ...
    @property
    def ScatterNSCRays(self) -> bool: ...
    @ScatterNSCRays.setter
    def ScatterNSCRays(self, value: bool) -> None: ...
    @property
    def SurfaceLineThickness(self) -> LineThicknessOptions: ...
    @SurfaceLineThickness.setter
    def SurfaceLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def RaysLineThickness(self) -> LineThicknessOptions: ...
    @RaysLineThickness.setter
    def RaysLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def ConfigurationAll(self) -> bool: ...
    @ConfigurationAll.setter
    def ConfigurationAll(self, value: bool) -> None: ...
    @property
    def ConfigurationCurrent(self) -> bool: ...
    @ConfigurationCurrent.setter
    def ConfigurationCurrent(self, value: bool) -> None: ...
    @property
    def ConfigurationOffsetX(self) -> float: ...
    @ConfigurationOffsetX.setter
    def ConfigurationOffsetX(self, value: float) -> None: ...
    @property
    def ConfigurationOffsetY(self) -> float: ...
    @ConfigurationOffsetY.setter
    def ConfigurationOffsetY(self, value: float) -> None: ...
    @property
    def ConfigurationOffsetZ(self) -> float: ...
    @ConfigurationOffsetZ.setter
    def ConfigurationOffsetZ(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleX(self) -> float: ...
    @CameraViewpointAngleX.setter
    def CameraViewpointAngleX(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleY(self) -> float: ...
    @CameraViewpointAngleY.setter
    def CameraViewpointAngleY(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleZ(self) -> float: ...
    @CameraViewpointAngleZ.setter
    def CameraViewpointAngleZ(self, value: float) -> None: ...
    @property
    def ImageExportData(self) -> IImageExportData: ...
    @property
    def DetectorDisplayMode(self) -> DetectorDisplayModeOptions: ...
    @DetectorDisplayMode.setter
    def DetectorDisplayMode(self, value: DetectorDisplayModeOptions) -> None: ...
    @property
    def Opacity(self) -> OpacityOptions: ...
    @Opacity.setter
    def Opacity(self, value: OpacityOptions) -> None: ...
    @property
    def Background(self) -> BackgroundOptions: ...
    @Background.setter
    def Background(self, value: BackgroundOptions) -> None: ...
    @property
    def Brightness(self) -> BrightnessOptions: ...
    @Brightness.setter
    def Brightness(self, value: BrightnessOptions) -> None: ...
    def SetCameraViewpoint(self, cameraViewpoint: CameraViewpointOptions) -> None: ...
    def SetConfigurationEnabled(self, config: int, enabled: bool) -> None: ...

class IShadedModelExport(ISystemTool):
    @property
    def SaveImageAsFile(self) -> bool: ...
    @SaveImageAsFile.setter
    def SaveImageAsFile(self, value: bool) -> None: ...
    @property
    def OutputFileName(self) -> str: ...
    @OutputFileName.setter
    def OutputFileName(self, value: str) -> None: ...
    @property
    def IsValidFileName(self) -> bool: ...
    @property
    def OutputPixelWidth(self) -> int: ...
    @OutputPixelWidth.setter
    def OutputPixelWidth(self, value: int) -> None: ...
    @property
    def OutputPixelHeight(self) -> int: ...
    @OutputPixelHeight.setter
    def OutputPixelHeight(self, value: int) -> None: ...
    @property
    def StartSurface(self) -> int: ...
    @StartSurface.setter
    def StartSurface(self, value: int) -> None: ...
    @property
    def EndSurface(self) -> int: ...
    @EndSurface.setter
    def EndSurface(self, value: int) -> None: ...
    @property
    def NumberOfRays(self) -> int: ...
    @NumberOfRays.setter
    def NumberOfRays(self, value: int) -> None: ...
    @property
    def ColorRaysBy(self) -> ColorRaysByOptions: ...
    @ColorRaysBy.setter
    def ColorRaysBy(self, value: ColorRaysByOptions) -> None: ...
    @property
    def RayPattern(self) -> RayPatternType: ...
    @RayPattern.setter
    def RayPattern(self, value: RayPatternType) -> None: ...
    @property
    def Wavelength(self) -> int: ...
    @Wavelength.setter
    def Wavelength(self, value: int) -> None: ...
    @property
    def Field(self) -> int: ...
    @Field.setter
    def Field(self, value: int) -> None: ...
    @property
    def Opacity(self) -> OpacityOptions: ...
    @Opacity.setter
    def Opacity(self, value: OpacityOptions) -> None: ...
    @property
    def Background(self) -> BackgroundOptions: ...
    @Background.setter
    def Background(self, value: BackgroundOptions) -> None: ...
    @property
    def Brightness(self) -> BrightnessOptions: ...
    @Brightness.setter
    def Brightness(self, value: BrightnessOptions) -> None: ...
    @property
    def AngularSegments(self) -> NumberSegmentsOptions: ...
    @AngularSegments.setter
    def AngularSegments(self, value: NumberSegmentsOptions) -> None: ...
    @property
    def RadialSegments(self) -> NumberSegmentsOptions: ...
    @RadialSegments.setter
    def RadialSegments(self, value: NumberSegmentsOptions) -> None: ...
    @property
    def FletchRays(self) -> bool: ...
    @FletchRays.setter
    def FletchRays(self, value: bool) -> None: ...
    @property
    def DeleteVignetted(self) -> bool: ...
    @DeleteVignetted.setter
    def DeleteVignetted(self, value: bool) -> None: ...
    @property
    def SplitNSCRays(self) -> bool: ...
    @SplitNSCRays.setter
    def SplitNSCRays(self, value: bool) -> None: ...
    @property
    def ScatterNSCRays(self) -> bool: ...
    @ScatterNSCRays.setter
    def ScatterNSCRays(self, value: bool) -> None: ...
    @property
    def SurfaceLineThickness(self) -> LineThicknessOptions: ...
    @SurfaceLineThickness.setter
    def SurfaceLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def RaysLineThickness(self) -> LineThicknessOptions: ...
    @RaysLineThickness.setter
    def RaysLineThickness(self, value: LineThicknessOptions) -> None: ...
    @property
    def ImageExportData(self) -> IImageExportData: ...
    @property
    def ConfigurationAll(self) -> bool: ...
    @ConfigurationAll.setter
    def ConfigurationAll(self, value: bool) -> None: ...
    @property
    def ConfigurationCurrent(self) -> bool: ...
    @ConfigurationCurrent.setter
    def ConfigurationCurrent(self, value: bool) -> None: ...
    @property
    def ConfigurationOffsetX(self) -> float: ...
    @ConfigurationOffsetX.setter
    def ConfigurationOffsetX(self, value: float) -> None: ...
    @property
    def ConfigurationOffsetY(self) -> float: ...
    @ConfigurationOffsetY.setter
    def ConfigurationOffsetY(self, value: float) -> None: ...
    @property
    def ConfigurationOffsetZ(self) -> float: ...
    @ConfigurationOffsetZ.setter
    def ConfigurationOffsetZ(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleX(self) -> float: ...
    @CameraViewpointAngleX.setter
    def CameraViewpointAngleX(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleY(self) -> float: ...
    @CameraViewpointAngleY.setter
    def CameraViewpointAngleY(self, value: float) -> None: ...
    @property
    def CameraViewpointAngleZ(self) -> float: ...
    @CameraViewpointAngleZ.setter
    def CameraViewpointAngleZ(self, value: float) -> None: ...
    @property
    def DrawSection(self) -> DrawSectionOptions: ...
    @DrawSection.setter
    def DrawSection(self, value: DrawSectionOptions) -> None: ...
    def SetCameraViewpoint(self, cameraViewpoint: CameraViewpointOptions) -> None: ...
    def SetConfigurationEnabled(self, config: int, enabled: bool) -> None: ...
    def SetFieldsAll(self) -> None: ...
    def SetWavelengthsAll(self) -> None: ...

class LineThicknessOptions:
    Standard = 0
    Thick = 1
    Thickest = 2
    Thinnest = -2
    Thin = -1

class NumberSegmentsOptions:
    S_8 = 0
    S_16 = 1
    S_32 = 2
    S_64 = 3
    S_128 = 4

class OpacityOptions:
    Ignore = 0
    Consider = 1
    All50Percent = 2

class RayTraceOptions:
    UseRays = 0
    LightningTraceTrueColor = 1
    LightningTraceAvgWavelength = 2

class RealPupilOptions:
    Pupils_Off = 0
    Pupils_4 = 4
    Pupils_8 = 8
    Pupils_16 = 16
    Pupils_32 = 32



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations
from . import FileManager, General, Layouts, Optimization, OptimizationTools, RayTrace, Tolerancing

__all__ = (
    "FileManager",
    "General",
    "Layouts",
    "Optimization",
    "OptimizationTools",
    "RayTrace",
    "Tolerancing",
    "CriticalRayType",
    "HPCEnvironments",
    "HPCNodeSize",
    "HPCRunState",
    "MaterialFormulas",
    "MaterialStatuses",
    "RayPatternOption",
    "RunStatus",
    "VertexOrder",
)

class CriticalRayType:
    Chief: CriticalRayType = None
    Marginal: CriticalRayType = None
    Grid: CriticalRayType = None
    Ring: CriticalRayType = None
    Y_Fan: CriticalRayType = None
    X_Fan: CriticalRayType = None
    XY_Fan: CriticalRayType = None
    List: CriticalRayType = None

class HPCEnvironments:
    OnPremise: HPCEnvironments = None
    AWSKubernetes: HPCEnvironments = None
    AzureKubernetes: HPCEnvironments = None

class HPCNodeSize:
    Default: HPCNodeSize = None
    Tiny: HPCNodeSize = None
    Small: HPCNodeSize = None
    Medium: HPCNodeSize = None
    Large: HPCNodeSize = None
    XLarge: HPCNodeSize = None

class HPCRunState:
    NotRunning: HPCRunState = None
    Initializing: HPCRunState = None
    ClusterAllocating: HPCRunState = None
    UploadingData: HPCRunState = None
    Queued: HPCRunState = None
    RunStarting: HPCRunState = None
    WaitingForResults: HPCRunState = None
    Complete: HPCRunState = None

class MaterialFormulas:
    Schott: MaterialFormulas = None
    Sellmeier1: MaterialFormulas = None
    Herzberger: MaterialFormulas = None
    Sellmeier2: MaterialFormulas = None
    Conrady: MaterialFormulas = None
    Sellmeier3: MaterialFormulas = None
    Handbook1: MaterialFormulas = None
    Handbook2: MaterialFormulas = None
    Sellmeier4: MaterialFormulas = None
    Extended: MaterialFormulas = None
    Sellmeier5: MaterialFormulas = None
    Extended2: MaterialFormulas = None
    Extended3: MaterialFormulas = None

class MaterialStatuses:
    Standard: MaterialStatuses = None
    Preferred: MaterialStatuses = None
    Obsolete: MaterialStatuses = None
    Special: MaterialStatuses = None
    Melt: MaterialStatuses = None

class RayPatternOption:
    XyFan: RayPatternOption = None
    XFan: RayPatternOption = None
    YFan: RayPatternOption = None
    ChiefAndRing: RayPatternOption = None
    List: RayPatternOption = None
    Grid: RayPatternOption = None
    ChiefAndMarginals: RayPatternOption = None

class RunStatus:
    Completed: RunStatus = None
    FailedToStart: RunStatus = None
    TimedOut: RunStatus = None
    InvalidTimeout: RunStatus = None

class VertexOrder:
    First: VertexOrder = None
    Second: VertexOrder = None
    Third: VertexOrder = None



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis.Settings import IAS_

__all__ = ("IAS_PathAnalysis", "PathAnalysisSortType")

class IAS_PathAnalysis(IAS_):
    @property
    def RayDatabaseFile(self) -> str: ...
    @RayDatabaseFile.setter
    def RayDatabaseFile(self, value: str) -> None: ...
    @property
    def AvailableRayDatabaseFiles(self) -> list[str]: ...
    @property
    def FirstRay(self) -> int: ...
    @FirstRay.setter
    def FirstRay(self, value: int) -> None: ...
    @property
    def LastRay(self) -> int: ...
    @LastRay.setter
    def LastRay(self, value: int) -> None: ...
    @property
    def FilterString(self) -> str: ...
    @FilterString.setter
    def FilterString(self, value: str) -> None: ...
    @property
    def GeneratePathFilters(self) -> bool: ...
    @GeneratePathFilters.setter
    def GeneratePathFilters(self, value: bool) -> None: ...
    @property
    def RelativeMinimumFlux(self) -> float: ...
    @RelativeMinimumFlux.setter
    def RelativeMinimumFlux(self, value: float) -> None: ...
    @property
    def SortBy(self) -> PathAnalysisSortType: ...
    @SortBy.setter
    def SortBy(self, value: PathAnalysisSortType) -> None: ...

class PathAnalysisSortType:
    TotalEndingFlux = 0
    NumberOfObjectsStruck = 1
    NumberOfBranches = 2
    LastObjectStruck = 3
    NumberOfUniqueObjectsStruct = 4
    AverageOpticalPathLength = 5
    NumberOfDiffractionEvents = 6



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from typing import overload

from zospy.api._ZOSAPI.Analysis.Data import IAR_
from zospy.api._ZOSAPI.Analysis.Settings import IAS_
from zospy.api._ZOSAPI.Common import ZemaxColor

from . import Data, PhysicalOptics, RayTracing, Settings, Tolerancing

__all__ = (
    "Data",
    "PhysicalOptics",
    "RayTracing",
    "Settings",
    "Tolerancing",
    "AnalysisIDM",
    "Beam",
    "BestFitSphereOptions",
    "ColorPaletteType",
    "DetectorViewerShowAsTypes",
    "DetectorViewerShowDataTypes",
    "ErrorType",
    "GiaShowAsTypes",
    "GridPlotType",
    "HuygensShowAsTypes",
    "HuygensSurfaceMftShowAsTypes",
    "I_Analyses",
    "IA_",
    "IColorTranslator",
    "IMessage",
    "IMessages",
    "IUser2DLineData",
    "IUserAnalysisData",
    "IUserGridData",
    "IUserGridRGBData",
    "IUserTextData",
    "POPSampling",
    "RemoveOptions",
    "SampleSizes",
    "SampleSizes_ContrastLoss",
    "SampleSizes_Pow2Plus1",
    "SampleSizes_Pow2Plus1_X",
    "ShowAs",
    "SurfaceCurvatureCrossData",
    "SurfaceCurvatureData",
    "SurfacePhaseData",
    "SurfacePhaseSlopeCrossData",
    "SurfacePhaseSlopeData",
    "SurfaceSagData",
    "SurfaceSlopeCrossData",
    "SurfaceSlopeData",
    "UserAnalysisDataType",
)

class AnalysisIDM:
    RayFan = 0
    OpticalPathFan = 1
    PupilAberrationFan = 2
    FieldCurvatureAndDistortion = 3
    FocalShiftDiagram = 4
    GridDistortion = 5
    LateralColor = 6
    LongitudinalAberration = 7
    RayTrace = 8
    SeidelCoefficients = 9
    SeidelDiagram = 10
    ZernikeAnnularCoefficients = 11
    ZernikeCoefficientsVsField = 12
    ZernikeFringeCoefficients = 13
    ZernikeStandardCoefficients = 14
    FftMtf = 15
    FftThroughFocusMtf = 16
    GeometricThroughFocusMtf = 17
    GeometricMtf = 18
    FftMtfMap = 19
    GeometricMtfMap = 20
    FftSurfaceMtf = 21
    FftMtfvsField = 22
    GeometricMtfvsField = 23
    HuygensMtfvsField = 24
    HuygensMtf = 25
    HuygensSurfaceMtf = 26
    HuygensThroughFocusMtf = 27
    FftPsf = 28
    FftPsfCrossSection = 29
    FftPsfLineEdgeSpread = 30
    HuygensPsfCrossSection = 31
    HuygensPsf = 32
    DiffractionEncircledEnergy = 33
    GeometricEncircledEnergy = 34
    GeometricLineEdgeSpread = 35
    ExtendedSourceEncircledEnergy = 36
    SurfaceCurvatureCross = 37
    SurfacePhaseCross = 38
    SurfaceSagCross = 39
    SurfaceCurvature = 40
    SurfacePhase = 41
    SurfaceSag = 42
    StandardSpot = 43
    ThroughFocusSpot = 44
    FullFieldSpot = 45
    MatrixSpot = 46
    ConfigurationMatrixSpot = 47
    RMSField = 48
    RMSFieldMap = 49
    RMSLambdaDiagram = 50
    RMSFocus = 51
    Foucault = 52
    Interferogram = 53
    WavefrontMap = 54
    DetectorViewer = 55
    Draw2D = 56
    Draw3D = 57
    ImageSimulation = 58
    GeometricImageAnalysis = 59
    IMABIMFileViewer = 60
    GeometricBitmapImageAnalysis = 61
    BitmapFileViewer = 62
    LightSourceAnalysis = 63
    PartiallyCoherentImageAnalysis = 64
    ExtendedDiffractionImageAnalysis = 65
    BiocularFieldOfViewAnalysis = 66
    BiocularDipvergenceConvergence = 67
    RelativeIllumination = 68
    VignettingDiagramSettings = 69
    FootprintSettings = 70
    YYbarDiagram = 71
    PowerFieldMapSettings = 72
    PowerPupilMapSettings = 73
    IncidentAnglevsImageHeight = 74
    FiberCouplingSettings = 75
    YNIContributions = 76
    SagTable = 77
    CardinalPoints = 78
    DispersionDiagram = 79
    GlassMap = 80
    AthermalGlassMap = 81
    InternalTransmissionvsWavelength = 82
    DispersionvsWavelength = 83
    GrinProfile = 84
    GradiumProfile = 85
    UniversalPlot1D = 86
    UniversalPlot2D = 87
    PolarizationRayTrace = 88
    PolarizationPupilMap = 89
    Transmission = 90
    PhaseAberration = 91
    TransmissionFan = 92
    ParaxialGaussianBeam = 93
    SkewGaussianBeam = 94
    PhysicalOpticsPropagation = 95
    BeamFileViewer = 96
    ReflectionvsAngle = 97
    TransmissionvsAngle = 98
    AbsorptionvsAngle = 99
    DiattenuationvsAngle = 100
    PhasevsAngle = 101
    RetardancevsAngle = 102
    ReflectionvsWavelength = 103
    TransmissionvsWavelength = 104
    AbsorptionvsWavelength = 105
    DiattenuationvsWavelength = 106
    PhasevsWavelength = 107
    RetardancevsWavelength = 108
    DirectivityPlot = 109
    SourcePolarViewer = 110
    PhotoluminscenceViewer = 111
    SourceSpectrumViewer = 112
    RadiantSourceModelViewerSettings = 113
    SurfaceDataSettings = 114
    PrescriptionDataSettings = 115
    FileComparatorSettings = 116
    PartViewer = 117
    ReverseRadianceAnalysis = 118
    PathAnalysis = 119
    FluxvsWavelength = 120
    RoadwayLighting = 121
    SourceIlluminationMap = 122
    ScatterFunctionViewer = 123
    ScatterPolarPlotSettings = 124
    ZemaxElementDrawing = 125
    ShadedModel = 126
    NSCShadedModel = 127
    NSC3DLayout = 128
    NSCObjectViewer = 129
    RayDatabaseViewer = 130
    ISOElementDrawing = 131
    SystemData = 132
    TestPlateList = 133
    SourceColorChart1931 = 134
    SourceColorChart1976 = 135
    PrescriptionGraphic = 136
    CriticalRayTracer = 137
    ContrastLoss = 138
    CoatingListing = 139
    FullFieldAberration = 140
    SurfaceSlope = 141
    SurfaceSlopeCross = 142
    QuickYield = 143
    SystemCheck = 144
    ToleranceYield = 145
    ToleranceHistogram = 146
    DiffEfficiency2D = 147
    DiffEfficiencyAngular = 148
    DiffEfficiencyChromatic = 149
    NSCSurfaceSag = 150
    NSCSingleRayTrace = 151
    NSCGeometricMtf = 152
    SurfacePhaseSlope = 153
    SurfacePhaseSlopeCross = 154
    XXXTemplateXXX = -1

class Beam:
    Reference = 0
    Configuration_1 = 1
    Configuration_2 = 2
    Configuration_3 = 3
    Configuration_4 = 4
    Configuration_5 = 5
    Configuration_6 = 6
    Configuration_7 = 7
    Configuration_8 = 8
    Configuration_9 = 9
    Configuration_10 = 10
    Configuration_11 = 11
    Configuration_12 = 12
    Configuration_13 = 13
    Configuration_14 = 14
    Configuration_15 = 15
    Configuration_16 = 16
    Configuration_17 = 17
    Configuration_18 = 18
    Configuration_19 = 19
    Configuration_20 = 20

class BestFitSphereOptions:
    MinimumVolume = 0
    MinimumRMS = 1
    MinimumRMSWithOffset = 2

class ColorPaletteType:
    GreyScale = 0
    FalseColor = 1
    FalseColorOriginal = 2
    Viridis = 3
    Magma = 4

class DetectorViewerShowAsTypes:
    GreyScale = 0
    FullListing = 0
    Text_CrossSection_Row = 1
    AzimuthCrossSection = 1
    GreyScale_Inverted = 1
    Text_CrossSection_Column = 2
    FalseColor = 2
    FluxVsWaveLength = 3
    FalseColor_Inverted = 3
    TrueColor = 4
    CrossSection_Row = 4
    CrossSection = 5
    Color_CrossSection_Row = 5
    CrossSection_Column = 5
    Directivity_Full = 6
    Color_CrossSection_Column = 6
    GeometricMtf = 6
    Color_FluxVsWavelength = 7
    Directivity_Half = 7

class DetectorViewerShowDataTypes:
    IncoherentIrradiance = 0
    Polar_AngleSpace = 0
    IncoherentFluence = 0
    IncidentFlux = 0
    PositionSpace = 0
    IncoherentIlluminance = 0
    CoherentFluence = 1
    AngleSpace = 1
    CoherentIrradiance = 1
    AbsorbedFlux = 1
    CoherentIlluminance = 1
    AbsorbedFluxVolume = 2
    CoherentPhase = 2
    RadiantIntensity = 3
    LuminousIntensity = 3
    LuminancePositionSpace = 4
    RadiancePositionSpace = 4
    LuminanceAngleSpace = 5
    RadianceAngleSpace = 5

class ErrorType:
    Success = 0
    InvalidParameter = 1
    InvalidSettings = 2
    Failed = 3
    AnalysisUnavailableForProgramMode = 4
    NotYetImplemented = 5
    NoSolverLicenseAvailable = 10000
    ToolAlreadyOpen = 10001
    SequentialOnly = 10002
    NonSequentialOnly = 10003
    SingleNSCRayTraceSupported = 10004
    HPCNotAvailable = 10005
    FeatureNotSupported = 10006
    NotAvailableInLegacy = 10007

class GiaShowAsTypes:
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5
    SpotDiagram = 6
    CrossX = 7
    CrossY = 8

class GridPlotType:
    Surface = 0
    Contour = 1
    GrayScale = 2
    InverseGrayScale = 3
    FalseColor = 4
    InverseFalseColor = 5

class HuygensShowAsTypes:
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5
    TrueColor = 6

class HuygensSurfaceMftShowAsTypes:
    GreyScale = 0
    InverseGreyScale = 1
    FalseColor = 2
    InverseFalseColor = 3

class I_Analyses:
    @property
    def NumberOfAnalyses(self) -> int: ...
    @overload
    def CloseAnalysis(self, index: int) -> bool: ...
    @overload
    def CloseAnalysis(self, analysis: IA_) -> bool: ...
    def CreateColorTranslatorAuto(
        self, palette: ColorPaletteType, inversePalette: bool, numberOfShades: int
    ) -> IColorTranslator: ...
    def CreateColorTranslatorFixed(
        self, minValue: float, maxValue: float, palette: ColorPaletteType, inversePalette: bool, numberOfShades: int
    ) -> IColorTranslator: ...
    def CreateLogColorTranslatorAuto(
        self, palette: ColorPaletteType, inversePalette: bool, logBase: float, numberOfShades: int
    ) -> IColorTranslator: ...
    def CreateLogColorTranslatorFixed(
        self,
        minValue: float,
        maxValue: float,
        palette: ColorPaletteType,
        inversePalette: bool,
        logBase: float,
        numberOfShades: int,
    ) -> IColorTranslator: ...
    def Get_AnalysisAtIndex(self, index: int) -> IA_: ...
    def New_AbsorptionVsAngle(self) -> IA_: ...
    def New_AbsorptionVsWavelength(self) -> IA_: ...
    def New_Analysis(self, AnalysisType: AnalysisIDM) -> IA_: ...
    def New_Analysis_SettingsFirst(self, AnalysisType: AnalysisIDM) -> IA_: ...
    def New_ConfigurationMatrixSpot(self) -> IA_: ...
    def New_ContrastLossMap(self) -> IA_: ...
    def New_CriticalRayTracer(self) -> IA_: ...
    def New_DetectorViewer(self) -> IA_: ...
    def New_DiattenuationVsAngle(self) -> IA_: ...
    def New_DiattenuationVsWavelength(self) -> IA_: ...
    def New_DiffractionEncircledEnergy(self) -> IA_: ...
    def New_ExtendedSourceEncircledEnergy(self) -> IA_: ...
    def New_FftMtf(self) -> IA_: ...
    def New_FftMtfMap(self) -> IA_: ...
    def New_FftMtfvsField(self) -> IA_: ...
    def New_FftPsf(self) -> IA_: ...
    def New_FftPsfCrossSection(self) -> IA_: ...
    def New_FftPsfLineEdgeSpread(self) -> IA_: ...
    def New_FftSurfaceMtf(self) -> IA_: ...
    def New_FftThroughFocusMtf(self) -> IA_: ...
    def New_FieldCurvatureAndDistortion(self) -> IA_: ...
    def New_FileComparator(self) -> IA_: ...
    def New_FocalShiftDiagram(self) -> IA_: ...
    def New_Foucault(self) -> IA_: ...
    def New_FullFieldAberration(self) -> IA_: ...
    def New_FullFieldSpot(self) -> IA_: ...
    def New_GeometricEncircledEnergy(self) -> IA_: ...
    def New_GeometricImageAnalysis(self) -> IA_: ...
    def New_GeometricLineEdgeSpread(self) -> IA_: ...
    def New_GeometricMtf(self) -> IA_: ...
    def New_GeometricMtfMap(self) -> IA_: ...
    def New_GeometricMtfvsField(self) -> IA_: ...
    def New_GeometricThroughFocusMtf(self) -> IA_: ...
    def New_GridDistortion(self) -> IA_: ...
    def New_GrinProfile(self) -> IA_: ...
    def New_HuygensMtf(self) -> IA_: ...
    def New_HuygensMtfvsField(self) -> IA_: ...
    def New_HuygensPsf(self) -> IA_: ...
    def New_HuygensPsfCrossSection(self) -> IA_: ...
    def New_HuygensSurfaceMtf(self) -> IA_: ...
    def New_HuygensThroughFocusMtf(self) -> IA_: ...
    def New_ImageSimulation(self) -> IA_: ...
    def New_Interferogram(self) -> IA_: ...
    def New_InternalTransmissionvsWavelength(self) -> IA_: ...
    def New_LateralColor(self) -> IA_: ...
    def New_LongitudinalAberration(self) -> IA_: ...
    def New_MatrixSpot(self) -> IA_: ...
    def New_NSCGeometricMtf(self) -> IA_: ...
    def New_NSCSingleRayTrace(self) -> IA_: ...
    def New_NSCSurfaceSag(self) -> IA_: ...
    def New_OpticalPathFan(self) -> IA_: ...
    def New_PathAnalysis(self) -> IA_: ...
    def New_PhaseVsAngle(self) -> IA_: ...
    def New_PhaseVsWavelength(self) -> IA_: ...
    def New_PupilAberrationFan(self) -> IA_: ...
    def New_QuickYield(self) -> IA_: ...
    def New_RayFan(self) -> IA_: ...
    def New_RayTrace(self) -> IA_: ...
    def New_ReflectionVsAngle(self) -> IA_: ...
    def New_ReflectionVsWavelength(self) -> IA_: ...
    def New_RelativeIllumination(self) -> IA_: ...
    def New_RetardanceVsAngle(self) -> IA_: ...
    def New_RetardanceVsWavelength(self) -> IA_: ...
    def New_RMSField(self) -> IA_: ...
    def New_RMSFieldMap(self) -> IA_: ...
    def New_RMSFocus(self) -> IA_: ...
    def New_RMSLambdaDiagram(self) -> IA_: ...
    def New_SeidelCoefficients(self) -> IA_: ...
    def New_SeidelDiagram(self) -> IA_: ...
    def New_StandardSpot(self) -> IA_: ...
    def New_SurfaceCurvature(self) -> IA_: ...
    def New_SurfaceCurvatureCross(self) -> IA_: ...
    def New_SurfacePhase(self) -> IA_: ...
    def New_SurfacePhaseCross(self) -> IA_: ...
    def New_SurfacePhaseSlope(self) -> IA_: ...
    def New_SurfacePhaseSlopeCross(self) -> IA_: ...
    def New_SurfaceSag(self) -> IA_: ...
    def New_SurfaceSagCross(self) -> IA_: ...
    def New_SurfaceSlope(self) -> IA_: ...
    def New_SurfaceSlopeCross(self) -> IA_: ...
    def New_ThroughFocusSpot(self) -> IA_: ...
    def New_TolerancingHistogram(self) -> IA_: ...
    def New_TolerancingYield(self) -> IA_: ...
    def New_TransmissionVsAngle(self) -> IA_: ...
    def New_TransmissionVsWavelength(self) -> IA_: ...
    def New_WavefrontMap(self) -> IA_: ...
    def New_XXXTemplateXXX(self) -> IA_: ...
    def New_ZernikeAnnularCoefficients(self) -> IA_: ...
    def New_ZernikeCoefficientsVsField(self) -> IA_: ...
    def New_ZernikeFringeCoefficients(self) -> IA_: ...
    def New_ZernikeStandardCoefficients(self) -> IA_: ...
    def RunHighSamplingPOP(
        self,
        configFile: str,
        xSampling: POPSampling,
        ySampling: POPSampling,
        outputTextFile: str,
        outputBeamFileName: str,
        saveBeamAtAllSurfaces: bool,
    ) -> IMessage: ...

class IA_:
    @property
    def Title(self) -> str: ...
    @property
    def GetAnalysisName(self) -> str: ...
    @property
    def AnalysisType(self) -> AnalysisIDM: ...
    @property
    def StatusMessages(self) -> IMessages: ...
    @property
    def HasAnalysisSpecificSettings(self) -> bool: ...
    def Apply(self) -> IMessage: ...
    def ApplyAndWaitForCompletion(self) -> IMessage: ...
    def Close(self) -> None: ...
    def GetResults(self) -> IAR_: ...
    def GetSettings(self) -> IAS_: ...
    def IsRunning(self) -> bool: ...
    def Release(self) -> None: ...
    def Terminate(self) -> bool: ...
    def ToFile(self, Filename: str, showSettings: bool, verify: bool) -> None: ...
    def WaitForCompletion(self) -> None: ...

class IColorTranslator:
    @property
    def Palette(self) -> ColorPaletteType: ...
    @property
    def IsInversePalette(self) -> bool: ...
    @property
    def IsAutoScaled(self) -> bool: ...
    @property
    def NumberOfShades(self) -> int: ...
    @property
    def MinValue(self) -> float: ...
    @property
    def MaxValue(self) -> float: ...
    @property
    def IsLog(self) -> bool: ...
    @property
    def LogBase(self) -> float: ...
    @property
    def CanConvertSingleValue(self) -> bool: ...
    def GetRGB(self, fullSize: int, data: list[float]) -> tuple[list[int], list[int], list[int]]: ...
    def GetRGB2DFloatSafe(self, vals: list[list[float]]) -> list[list[list[float]]]: ...
    def GetRGB2DSafe(self, vals: list[list[float]]) -> list[list[list[int]]]: ...
    def GetRGBFloat(self, fullSize: int, data: list[float]) -> tuple[list[float], list[float], list[float]]: ...
    def GetRGBFloatSafe(self, vals: list[float]) -> list[list[float]]: ...
    def GetRGBSafe(self, vals: list[float]) -> list[list[int]]: ...
    def GetSingleRGB(self, x: float) -> tuple[bool, int, int, int]: ...
    def GetSingleRGBFloat(self, x: float) -> tuple[bool, float, float, float]: ...

class IMessage:
    @property
    def ErrorCode(self) -> ErrorType: ...
    @property
    def Text(self) -> str: ...

class IMessages:
    def AllToString(self) -> str: ...
    @overload
    def WriteLine(self, s: str, userV: int, settingsV: int) -> None: ...
    @overload
    def WriteLine(self, s: str, userV: bool, settingsV: bool) -> None: ...
    @overload
    def WriteLine(self, s: str, userV: float, settingsV: float) -> None: ...
    @overload
    def WriteLine(self, s: str, userV: str, settingsV: str) -> None: ...

class IUser2DLineData:
    @property
    def PlotDescription(self) -> str: ...
    @property
    def XLabel(self) -> str: ...
    @XLabel.setter
    def XLabel(self, value: str) -> None: ...
    @property
    def YLabel(self) -> str: ...
    @YLabel.setter
    def YLabel(self, value: str) -> None: ...
    @property
    def NumberOfXValues(self) -> int: ...
    @property
    def NumberOfYValues(self) -> int: ...
    @property
    def NumberOfSeries(self) -> int: ...
    @property
    def XAxisMin(self) -> float: ...
    @XAxisMin.setter
    def XAxisMin(self, value: float) -> None: ...
    @property
    def XAxisMinAuto(self) -> bool: ...
    @XAxisMinAuto.setter
    def XAxisMinAuto(self, value: bool) -> None: ...
    @property
    def XAxisMax(self) -> float: ...
    @XAxisMax.setter
    def XAxisMax(self, value: float) -> None: ...
    @property
    def XAxisMaxAuto(self) -> bool: ...
    @XAxisMaxAuto.setter
    def XAxisMaxAuto(self, value: bool) -> None: ...
    @property
    def XAxisLog(self) -> bool: ...
    @XAxisLog.setter
    def XAxisLog(self, value: bool) -> None: ...
    @property
    def XAxisReversed(self) -> bool: ...
    @XAxisReversed.setter
    def XAxisReversed(self, value: bool) -> None: ...
    @property
    def XAxisSymmetric(self) -> bool: ...
    @XAxisSymmetric.setter
    def XAxisSymmetric(self, value: bool) -> None: ...
    @property
    def YAxisMin(self) -> float: ...
    @YAxisMin.setter
    def YAxisMin(self, value: float) -> None: ...
    @property
    def YAxisMinAuto(self) -> bool: ...
    @YAxisMinAuto.setter
    def YAxisMinAuto(self, value: bool) -> None: ...
    @property
    def YAxisMax(self) -> float: ...
    @YAxisMax.setter
    def YAxisMax(self, value: float) -> None: ...
    @property
    def YAxisMaxAuto(self) -> bool: ...
    @YAxisMaxAuto.setter
    def YAxisMaxAuto(self, value: bool) -> None: ...
    @property
    def YAxisLog(self) -> bool: ...
    @YAxisLog.setter
    def YAxisLog(self, value: bool) -> None: ...
    @property
    def YAxisReversed(self) -> bool: ...
    @YAxisReversed.setter
    def YAxisReversed(self, value: bool) -> None: ...
    @property
    def YAxisSymmetric(self) -> bool: ...
    @YAxisSymmetric.setter
    def YAxisSymmetric(self, value: bool) -> None: ...
    def AddSeries(
        self, seriesName: str, seriesColor: ZemaxColor, numberOfYValues: int, yValues: list[float]
    ) -> None: ...
    def AddSeriesSafe(self, seriesName: str, seriesColor: ZemaxColor, yValues: list[float]) -> None: ...

class IUserAnalysisData:
    @property
    def AnalysisNumber(self) -> int: ...
    @property
    def PlotType(self) -> UserAnalysisDataType: ...
    @property
    def WindowTitle(self) -> str: ...
    @WindowTitle.setter
    def WindowTitle(self, value: str) -> None: ...
    @property
    def FeatureDescription(self) -> str: ...
    @FeatureDescription.setter
    def FeatureDescription(self, value: str) -> None: ...
    @property
    def HeaderData(self) -> list[str]: ...
    @HeaderData.setter
    def HeaderData(self, value: list[str]) -> None: ...
    @property
    def ShowLegend(self) -> bool: ...
    @ShowLegend.setter
    def ShowLegend(self, value: bool) -> None: ...
    @property
    def UserSettings(self) -> ISettingsData: ...
    @property
    def RunAnalysisOnSettingsClosed(self) -> bool: ...
    @RunAnalysisOnSettingsClosed.setter
    def RunAnalysisOnSettingsClosed(self, value: bool) -> None: ...
    def Make2DLinePlot(self, PlotDescription: str, numValues: int, xValues: list[float]) -> IUser2DLineData: ...
    def Make2DLinePlotSafe(self, PlotDescription: str, xValues: list[float]) -> IUser2DLineData: ...
    def MakeGridPlot(self, PlotDescription: str) -> IUserGridData: ...
    def MakeGridRGBPlot(self, PlotDescription: str) -> IUserGridRGBData: ...
    def MakeText(self) -> IUserTextData: ...

class IUserGridData:
    @property
    def PlotDescription(self) -> str: ...
    @property
    def XLabel(self) -> str: ...
    @XLabel.setter
    def XLabel(self, value: str) -> None: ...
    @property
    def YLabel(self) -> str: ...
    @YLabel.setter
    def YLabel(self, value: str) -> None: ...
    @property
    def ValueLabel(self) -> str: ...
    @ValueLabel.setter
    def ValueLabel(self, value: str) -> None: ...
    @property
    def ShowAsType(self) -> GridPlotType: ...
    @ShowAsType.setter
    def ShowAsType(self, value: GridPlotType) -> None: ...
    @property
    def NumberOfXDataValues(self) -> int: ...
    @property
    def XDataMin(self) -> float: ...
    @property
    def XDataMax(self) -> float: ...
    @property
    def DataDX(self) -> float: ...
    @property
    def NumberOfYDataValues(self) -> int: ...
    @property
    def YDataMin(self) -> float: ...
    @property
    def YDataMax(self) -> float: ...
    @property
    def DataDY(self) -> float: ...
    @property
    def XAxisMin(self) -> float: ...
    @XAxisMin.setter
    def XAxisMin(self, value: float) -> None: ...
    @property
    def XAxisMinAuto(self) -> bool: ...
    @XAxisMinAuto.setter
    def XAxisMinAuto(self, value: bool) -> None: ...
    @property
    def XAxisMax(self) -> float: ...
    @XAxisMax.setter
    def XAxisMax(self, value: float) -> None: ...
    @property
    def XAxisMaxAuto(self) -> bool: ...
    @XAxisMaxAuto.setter
    def XAxisMaxAuto(self, value: bool) -> None: ...
    @property
    def XAxisSymmetric(self) -> bool: ...
    @XAxisSymmetric.setter
    def XAxisSymmetric(self, value: bool) -> None: ...
    @property
    def YAxisMin(self) -> float: ...
    @YAxisMin.setter
    def YAxisMin(self, value: float) -> None: ...
    @property
    def YAxisMinAuto(self) -> bool: ...
    @YAxisMinAuto.setter
    def YAxisMinAuto(self, value: bool) -> None: ...
    @property
    def YAxisMax(self) -> float: ...
    @YAxisMax.setter
    def YAxisMax(self, value: float) -> None: ...
    @property
    def YAxisMaxAuto(self) -> bool: ...
    @YAxisMaxAuto.setter
    def YAxisMaxAuto(self, value: bool) -> None: ...
    @property
    def YAxisSymmetric(self) -> bool: ...
    @YAxisSymmetric.setter
    def YAxisSymmetric(self, value: bool) -> None: ...
    @property
    def ZAxisMin(self) -> float: ...
    @ZAxisMin.setter
    def ZAxisMin(self, value: float) -> None: ...
    @property
    def ZAxisMinAuto(self) -> bool: ...
    @ZAxisMinAuto.setter
    def ZAxisMinAuto(self, value: bool) -> None: ...
    @property
    def ZAxisMax(self) -> float: ...
    @ZAxisMax.setter
    def ZAxisMax(self, value: float) -> None: ...
    @property
    def ZAxisMaxAuto(self) -> bool: ...
    @ZAxisMaxAuto.setter
    def ZAxisMaxAuto(self, value: bool) -> None: ...
    @property
    def ZAxisLog(self) -> bool: ...
    @ZAxisLog.setter
    def ZAxisLog(self, value: bool) -> None: ...
    @property
    def LogColorMap(self) -> bool: ...
    @LogColorMap.setter
    def LogColorMap(self, value: bool) -> None: ...
    @property
    def InterpolateLowResolutionContours(self) -> bool: ...
    @InterpolateLowResolutionContours.setter
    def InterpolateLowResolutionContours(self, value: bool) -> None: ...
    @property
    def XYAspectRatio(self) -> float: ...
    @XYAspectRatio.setter
    def XYAspectRatio(self, value: float) -> None: ...
    def SetData(self, totalSize: int, numXValues: int, numYValues: int, Data: list[float]) -> None: ...
    def SetDataSafe(self, Data: list[list[float]]) -> None: ...
    def SetXDataDimensions(self, xDataMin: float, xDataMax: float) -> None: ...
    def SetYDataDimensions(self, yDataMin: float, yDataMax: float) -> None: ...

class IUserGridRGBData:
    @property
    def PlotDescription(self) -> str: ...
    @property
    def XLabel(self) -> str: ...
    @XLabel.setter
    def XLabel(self, value: str) -> None: ...
    @property
    def YLabel(self) -> str: ...
    @YLabel.setter
    def YLabel(self, value: str) -> None: ...
    @property
    def ValueLabel(self) -> str: ...
    @ValueLabel.setter
    def ValueLabel(self, value: str) -> None: ...
    @property
    def NumberOfXDataValues(self) -> int: ...
    @property
    def XDataMin(self) -> float: ...
    @property
    def XDataMax(self) -> float: ...
    @property
    def DataDX(self) -> float: ...
    @property
    def NumberOfYDataValues(self) -> int: ...
    @property
    def YDataMin(self) -> float: ...
    @property
    def YDataMax(self) -> float: ...
    @property
    def DataDY(self) -> float: ...
    @property
    def XAxisMin(self) -> float: ...
    @XAxisMin.setter
    def XAxisMin(self, value: float) -> None: ...
    @property
    def XAxisMinAuto(self) -> bool: ...
    @XAxisMinAuto.setter
    def XAxisMinAuto(self, value: bool) -> None: ...
    @property
    def XAxisMax(self) -> float: ...
    @XAxisMax.setter
    def XAxisMax(self, value: float) -> None: ...
    @property
    def XAxisMaxAuto(self) -> bool: ...
    @XAxisMaxAuto.setter
    def XAxisMaxAuto(self, value: bool) -> None: ...
    @property
    def XAxisSymmetric(self) -> bool: ...
    @XAxisSymmetric.setter
    def XAxisSymmetric(self, value: bool) -> None: ...
    @property
    def YAxisMin(self) -> float: ...
    @YAxisMin.setter
    def YAxisMin(self, value: float) -> None: ...
    @property
    def YAxisMinAuto(self) -> bool: ...
    @YAxisMinAuto.setter
    def YAxisMinAuto(self, value: bool) -> None: ...
    @property
    def YAxisMax(self) -> float: ...
    @YAxisMax.setter
    def YAxisMax(self, value: float) -> None: ...
    @property
    def YAxisMaxAuto(self) -> bool: ...
    @YAxisMaxAuto.setter
    def YAxisMaxAuto(self, value: bool) -> None: ...
    @property
    def YAxisSymmetric(self) -> bool: ...
    @YAxisSymmetric.setter
    def YAxisSymmetric(self, value: bool) -> None: ...
    @property
    def XYAspectRatio(self) -> float: ...
    @XYAspectRatio.setter
    def XYAspectRatio(self, value: float) -> None: ...
    def SetData(self, fullSize: int, numXValues: int, numYValues: int, rgbData: list[float]) -> None: ...
    def SetDataRGB(
        self,
        fullSize: int,
        numXValues: int,
        numYValues: int,
        rData: list[float],
        gData: list[float],
        bData: list[float],
    ) -> None: ...
    def SetDataRGBSafe(self, rData: list[list[float]], gData: list[list[float]], bData: list[list[float]]) -> None: ...
    def SetDataSafe(self, rgbData: list[list[list[float]]]) -> None: ...
    def SetXDataDimensions(self, xDataMin: float, xDataMax: float) -> None: ...
    def SetYDataDimensions(self, yDataMin: float, yDataMax: float) -> None: ...

class IUserTextData:
    @property
    def Data(self) -> str: ...
    @Data.setter
    def Data(self, value: str) -> None: ...

class POPSampling:
    S_32 = 5
    S_64 = 6
    S_128 = 7
    S_256 = 8
    S_512 = 9
    S_1024 = 10
    S_1K = 10
    S_2048 = 11
    S_2K = 11
    S_4K = 12
    S_4096 = 12
    S_8192 = 13
    S_8K = 13
    S_16384 = 14
    S_16K = 14
    S_32K = 15
    S_32768 = 15
    S_65536 = 16
    S_64K = 16
    S_131072 = 17
    S_128K = 17
    S_256K = 18
    S_262144 = 18
    S_524288 = 19
    S_512K = 19
    S_1048576 = 20
    S_1M = 20
    S_2M = 21
    S_2097152 = 21
    S_4194304 = 22
    S_4M = 22
    S_8388608 = 23
    S_8M = 23
    S_16M = 24
    S_16777216 = 24
    S_33554432 = 25
    S_32M = 25
    S_67108864 = 26
    S_64M = 26
    S_128M = 27
    S_134217728 = 27
    S_268435456 = 28
    S_256M = 28
    S_536870912 = 29
    S_512M = 29
    S_1073741824 = 30
    S_1G = 30

class RemoveOptions:
    # None = 0
    BaseROC = 1
    BestFitSphere = 2
    BaseSag = 3
    CompositeSag = 4

class SampleSizes:
    S_32x32 = 1
    S_64x64 = 2
    S_128x128 = 3
    S_256x256 = 4
    S_512x512 = 5
    S_1024x1024 = 6
    S_2048x2048 = 7
    S_4096x4096 = 8
    S_8192x8192 = 9
    S_16384x16384 = 10

class SampleSizes_ContrastLoss:
    S_3x3 = 1
    S_5x5 = 2
    S_7x7 = 3
    S_9x9 = 4
    S_11x11 = 5
    S_13x13 = 6
    S_15x15 = 7
    S_17x17 = 8
    S_19x19 = 9
    S_21x21 = 10
    S_23x23 = 11
    S_25x25 = 12
    S_27x27 = 13
    S_29x29 = 14
    S_31x31 = 15
    S_33x33 = 16
    S_35x35 = 17
    S_37x37 = 18
    S_39x39 = 19
    S_41x41 = 20
    S_43x43 = 21
    S_45x45 = 22
    S_47x47 = 23
    S_49x49 = 24
    S_51x51 = 25
    S_53x53 = 26
    S_55x55 = 27
    S_57x57 = 28
    S_59x59 = 29
    S_61x61 = 30
    S_63x63 = 31
    S_65x65 = 32
    S_67x67 = 33
    S_69x69 = 34
    S_71x71 = 35
    S_73x73 = 36
    S_75x75 = 37
    S_77x77 = 38
    S_79x79 = 39
    S_81x81 = 40
    S_83x83 = 41

class SampleSizes_Pow2Plus1:
    S_33 = 1
    S_65 = 2
    S_129 = 3
    S_257 = 4
    S_513 = 5
    S_1025 = 6
    S_2049 = 7
    S_4097 = 8
    S_8193 = 9

class SampleSizes_Pow2Plus1_X:
    S_33x33 = 1
    S_65x65 = 2
    S_129x129 = 3
    S_257x257 = 4
    S_513x513 = 5
    S_1025x1025 = 6
    S_2049x2049 = 7
    S_4097x4097 = 8
    S_8193x8193 = 9

class ShowAs:
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5

class SurfaceCurvatureCrossData:
    TangentialCurvature = 0
    SagitalCurvature = 1
    X_Curvature = 2
    Y_Curvature = 3
    TanPlusSagCurvature = 4
    XPlusYCurvature = 5

class SurfaceCurvatureData:
    TangentialCurvature = 0
    SagitalCurvature = 1
    X_Curvature = 2
    Y_Curvature = 3

class SurfacePhaseData:
    SurfacePhase = 0

class SurfacePhaseSlopeCrossData:
    PhaseSlopeTangential = 0
    PhaseSlopeSaggital = 1
    PhaseSlopeX = 2
    PhaseSlopeY = 3
    PhaseSlopeTangentialAndSaggital = 4
    PhaseSlopeX_and_Y = 5

class SurfacePhaseSlopeData:
    PhaseSlopeTangential = 0
    PhaseSlopeSaggital = 1
    PhaseSlopeX = 2
    PhaseSlopeY = 3

class SurfaceSagData:
    SurfaceSag = 0

class SurfaceSlopeCrossData:
    TangentialCurvature = 0
    SagitalCurvature = 1
    X_Curvature = 2
    Y_Curvature = 3
    TanPlusSagCurvature = 4
    XPlusYCurvature = 5

class SurfaceSlopeData:
    SurfaceSlope = 0
    TangentialSlope = 0
    SagittalSlope = 1
    XSlope = 2
    YSlope = 3

class UserAnalysisDataType:
    # None = 0
    Line2D = 1
    Grid = 2
    GridRGB = 3
    Text = 4



import pytest
from pandas.testing import assert_frame_equal

from zospy.analyses.old.psf import huygens_psf

pytestmark = pytest.mark.old_analyses


class TestHuygensPSF:
    def test_can_run_huygens_psf(self, simple_system):
        result = huygens_psf(simple_system)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = huygens_psf(simple_system)

        assert result.from_json(result.to_json())

    def test_huygens_psf_returns_correct_result(self, simple_system, expected_data):
        result = huygens_psf(simple_system)

        assert_frame_equal(result.Data.astype(float), expected_data.Data.astype(float))

    def test_huygens_psf_matches_reference_data(self, simple_system, reference_data):
        result = huygens_psf(simple_system)

        assert_frame_equal(result.Data.astype(float), reference_data.Data.astype(float))

    def test_huygens_psf_asymmetric_returns_correct_result(self, decentered_system, expected_data):
        result = huygens_psf(decentered_system)

        assert_frame_equal(result.Data.astype(float), expected_data.Data.astype(float))

    def test_huygens_psf_asymmetric_matches_reference_data(self, decentered_system, reference_data):
        result = huygens_psf(decentered_system)

        assert_frame_equal(result.Data.astype(float), reference_data.Data.astype(float))



import pytest
from pandas.testing import assert_frame_equal

from zospy.analyses.extendedscene import GeometricImageAnalysis


class TestGeometricImageAnalysis:
    def test_can_run(self, simple_system):
        result = GeometricImageAnalysis().run(simple_system)
        assert result.data is not None

    def test_to_json(self, simple_system):
        result = GeometricImageAnalysis().run(simple_system)
        assert result.from_json(result.to_json()).to_json() == result.to_json()

    @pytest.mark.parametrize(
        "image_size,expected_minx,expected_miny",
        [
            (1, -0.5, -0.5),
            (1.4, -0.7, -0.7),
            (2.5, -1.25, -1.25),
            (20, -10, -10),
        ],
    )
    def test_geometric_image_analysis_returns_correct_origin(
        self, simple_system, image_size, expected_minx, expected_miny
    ):
        number_of_pixels = 100
        result = GeometricImageAnalysis(image_size=image_size, number_of_pixels=number_of_pixels).run(simple_system)

        pixel_width = image_size / number_of_pixels
        pixel_height = image_size / number_of_pixels

        # Adjust expected_minx and expected_miny with respectively 0.5 * pixel_width and 0.5 * pixel_height as the
        # columns and index from result.data provide the pixel center rather than the bottom left corner of the pixel,
        # while expected_minx and expected_miny point to that bottom left corner.
        assert result.data.columns[0] == (expected_minx + 0.5 * pixel_width)
        assert result.data.index[0] == (expected_miny + 0.5 * pixel_height)

    @pytest.mark.parametrize(
        "show_as,field_size,total_watts,rays_x_1000",
        [
            ("Surface", 0, 1, 100000),
            ("Surface", 10, 1, 100000),
            ("Surface", 10, 100, 100000),
            ("Surface", 0, 100, 100000),
            ("CrossX", 0, 1, 100000),
            ("CrossX", 10, 1, 100000),
            ("CrossX", 10, 100, 100000),
            ("CrossX", 0, 100, 100000),
        ],
    )
    def test_geometric_image_analysis_returns_correct_result(
        self, simple_system, show_as, field_size, total_watts, rays_x_1000, expected_data
    ):
        result = GeometricImageAnalysis(
            show_as=show_as, field_size=field_size, total_watts=total_watts, rays_x_1000=rays_x_1000
        ).run(simple_system)

        # Data in result.Data
        # rtol lower than normal due to randomness in calculated rays
        assert_frame_equal(result.data, expected_data.data, rtol=1e-2)

    @pytest.mark.parametrize(
        "show_as,field_size,total_watts,rays_x_1000",
        [
            ("Surface", 0, 1, 100000),
            ("Surface", 10, 1, 100000),
            ("Surface", 10, 100, 100000),
            ("Surface", 0, 100, 100000),
            ("CrossX", 0, 1, 100000),
            ("CrossX", 10, 1, 100000),
            ("CrossX", 10, 100, 100000),
            ("CrossX", 0, 100, 100000),
        ],
    )
    def test_geometric_image_analysis_matches_reference_data(
        self, simple_system, show_as, field_size, total_watts, rays_x_1000, reference_data
    ):
        result = GeometricImageAnalysis(
            show_as=show_as, field_size=field_size, total_watts=total_watts, rays_x_1000=rays_x_1000
        ).run(simple_system)

        # Data in result.Data
        # rtol lower than normal due to randomness in calculated rays
        assert_frame_equal(result.data, reference_data.data, rtol=1e-2)



import pytest

from tests.helpers import assert_dataclass_equal
from zospy.analyses.polarization import PolarizationPupilMap, PolarizationTransmission


class TestPolarizationTransmission:
    def test_can_run(self, simple_system):
        result = PolarizationTransmission().run(simple_system)
        assert result.data is not None

    def test_to_json(self, simple_system):
        result = PolarizationTransmission().run(simple_system)
        assert result.from_json(result.to_json()).to_json() == result.to_json()

    @pytest.mark.parametrize(
        "sampling,unpolarized,jx,jy,x_phase,y_phase",
        [
            ("32x32", False, 1, 0, 0, 0),
            ("32x32", False, 0, 1, 0, 0),
            ("32x32", False, 1, 1, 0, 0),
            ("32x32", False, 0.001, 1, 0, 0),
            ("64x64", False, 1, 1, 45, 90),
            ("64x64", True, 1, 0, 0, 0),
        ],
    )
    def test_transmission_returns_correct_result(
        self, polarized_system, sampling, unpolarized, jx, jy, x_phase, y_phase, expected_data
    ):
        result = PolarizationTransmission(
            sampling=sampling, unpolarized=unpolarized, jx=jx, jy=jy, x_phase=x_phase, y_phase=y_phase
        ).run(polarized_system)

        assert_dataclass_equal(result.data, expected_data.data)

    @pytest.mark.parametrize(
        "sampling,unpolarized,jx,jy,x_phase,y_phase",
        [
            ("32x32", False, 1, 0, 0, 0),
            ("32x32", False, 0, 1, 0, 0),
            ("32x32", False, 1, 1, 0, 0),
            ("32x32", False, 0.001, 1, 0, 0),
            ("64x64", False, 1, 1, 45, 90),
            ("64x64", True, 1, 0, 0, 0),
        ],
    )
    def test_transmission_matches_reference_data(
        self, polarized_system, sampling, unpolarized, jx, jy, x_phase, y_phase, reference_data
    ):
        result = PolarizationTransmission(
            sampling=sampling, unpolarized=unpolarized, jx=jx, jy=jy, x_phase=x_phase, y_phase=y_phase
        ).run(polarized_system)

        assert_dataclass_equal(result.data, reference_data.data)


XFAIL_REASON = "Intentionally skipped for this OpticStudio version. See https://zospy.readthedocs.io/compatibility."


class TestPolarizationPupilMap:
    def test_can_run(self, simple_system):
        result = PolarizationPupilMap().run(simple_system)
        assert result.data is not None

    def test_to_json(self, simple_system):
        result = PolarizationPupilMap().run(simple_system)
        assert result.from_json(result.to_json()).to_json() == result.to_json()

    @pytest.mark.parametrize(
        "jx,jy,x_phase,y_phase,surface,sampling",
        [
            (1, 0, 0, 0, "Image", "11x11"),
            (1, 1, 0, 0, 2, "11x11"),
            (0, 1, 0, 0, "Image", "11x11"),
            (1, 1, 45, 90, "Image", "17x17"),
        ],
    )
    def test_polarization_pupil_map_returns_correct_result(
        self, polarized_system, jx, jy, x_phase, y_phase, surface, sampling, expected_data
    ):
        result = PolarizationPupilMap(
            jx=jx, jy=jy, x_phase=x_phase, y_phase=y_phase, surface=surface, sampling=sampling
        ).run(polarized_system)

        assert_dataclass_equal(result.data, expected_data.data)

    @pytest.mark.parametrize(
        "jx,jy,x_phase,y_phase,surface,sampling",
        [
            pytest.param(
                1,
                0,
                0,
                0,
                "Image",
                "11x11",
                marks=pytest.mark.xfail_for_opticstudio_versions(["20.3.2"], XFAIL_REASON),
            ),
            (1, 1, 0, 0, 2, "11x11"),
            (0, 1, 0, 0, "Image", "11x11"),
            pytest.param(
                1,
                1,
                45,
                90,
                "Image",
                "17x17",
                marks=pytest.mark.xfail_for_opticstudio_versions(["20.3.2"], XFAIL_REASON),
            ),
        ],
    )
    def test_polarization_pupil_map_matches_reference_data(
        self, polarized_system, jx, jy, x_phase, y_phase, surface, sampling, reference_data
    ):
        result = PolarizationPupilMap(
            jx=jx, jy=jy, x_phase=x_phase, y_phase=y_phase, surface=surface, sampling=sampling
        ).run(polarized_system)

        assert_dataclass_equal(result.data, reference_data.data)



"""Nonsequential Shaded Model viewer."""

from __future__ import annotations

from typing import TYPE_CHECKING

from pydantic import Field

from zospy.analyses.decorators import analysis_settings
from zospy.analyses.systemviewers.base import ImageSize, SystemViewerWrapper

if TYPE_CHECKING:
    from zospy.api import _ZOSAPI

__all__ = ("NSCShadedModel", "NSCShadedModelSettings")


@analysis_settings
class NSCShadedModelSettings:
    """Settings for the nonsequential Shaded Model viewer.

    Notes
    -----
    Not all settings available in OpticStudio are supported yet.
    """

    image_size: ImageSize = Field(default=(800, 600), description="Image size")


class NSCShadedModel(SystemViewerWrapper[NSCShadedModelSettings], analysis_type="NSCShadedModel", mode="Nonsequential"):
    """Nonsequential Shaded Model viewer."""

    def __init__(self, *, image_size: ImageSize = (800, 600)):
        """Create a new nonsequential shaded model viewer.

        See Also
        --------
        NSCShadedModelSettings : Settings for the nonsequential shaded model viewer.
        """
        super().__init__(settings_kws=locals())

    def configure_layout_tool(self) -> _ZOSAPI.Tools.Layouts.IShadedModelExport:
        """Configure the nonsequential shaded model viewer."""
        layout_tool = self.oss.Tools.Layouts.OpenShadedModelExport()

        layout_tool.OutputPixelWidth, layout_tool.OutputPixelHeight = self.settings.image_size

        return layout_tool



import locale
import os
from pathlib import Path
from tempfile import mkstemp

import pytest

import zospy.api.config as _config
from zospy import constants
from zospy.analyses.base import new_analysis
from zospy.utils.pyutils import abspath, atox, xtoa
from zospy.utils.zputils import _get_number_field


@pytest.mark.parametrize("input_type", [str, Path])
class TestAbsPath:
    def test_abspath_returns_absolute_path(self, tmp_path, input_type, monkeypatch):
        filename = "test.txt"

        # Make sure path exists
        tmp_path.joinpath(filename).touch()

        # Change working directory to temporary directory
        monkeypatch.chdir(tmp_path)

        assert abspath(input_type(filename)) == str(tmp_path / filename)

    def test_abspath_raises_file_not_found_error(self, tmp_path, input_type, monkeypatch):
        filename = "test.txt"

        # Change working directory to temporary directory
        monkeypatch.chdir(tmp_path)

        with pytest.raises(FileNotFoundError):
            abspath(input_type(filename))

    def test_abspath_check_directory_only_does_not_raise_file_not_found_error(self, tmp_path, input_type, monkeypatch):
        filename = "test.txt"

        # Change working directory to temporary directory
        monkeypatch.chdir(tmp_path)

        assert abspath(input_type(filename), check_directory_only=True) == str(tmp_path / filename)

    def test_abspath_check_directory_only_raises_file_not_found_error(self, tmp_path, input_type, monkeypatch):
        filename = "non_existing_directory/test.txt"

        # Change working directory to temporary directory
        monkeypatch.chdir(tmp_path)

        with pytest.raises(FileNotFoundError):
            abspath(input_type(filename), check_directory_only=True)


class TestNumberToStringConversion:
    @pytest.mark.parametrize(
        "number,expected_output,decimal_point,thousands_separator",
        [
            (1.5, "1.5", ".", ""),
            (1.5, "1,5", ",", ""),
            (1.5, "1.5", ".", ","),
            (1.5, "1,5", ",", "."),
            (1.5, "1.5", ".", None),
            (1.5, "1,5", ",", None),
            (2, "2", ".", ","),
            (2, "2", ",", "."),
            (1234.5, "1.234,5", ",", "."),
            (1234.5, "1,234.5", ".", ","),
            (1234.5, "1234,5", ",", None),
            (1234.5, "1234.5", ".", None),
        ],
    )
    def test_xtoa_converts_number_correctly(self, number, expected_output, decimal_point, thousands_separator):
        result = xtoa(number, decimal_point=decimal_point, thousands_separator=thousands_separator)

        assert result == expected_output

    @pytest.mark.parametrize(
        "new_locale,number,expected_output",
        [
            ("de_CH", 1234.5, "1’234.5"),  # noqa: RUF001
            ("de_DE", 1234.5, "1.234,5"),
            ("en_US", 1234.5, "1,234.5"),
            ("nl_NL", 1234.5, "1.234,5"),
            ("zh_CN", 1234.5, "1,234.5"),
        ],
    )
    def test_xtoa_converts_number_correctly_for_different_locale_categories(
        self, new_locale, number, expected_output, monkeypatch
    ):
        if new_locale not in locale.windows_locale.values():
            pytest.skip(f"Locale '{new_locale}' not available.")

        # get new locale
        loc = locale.setlocale(locale.LC_NUMERIC)
        locale.setlocale(locale.LC_NUMERIC, new_locale)

        monkeypatch.setattr(_config, "THOUSANDS_SEPARATOR", locale.localeconv()["thousands_sep"])
        monkeypatch.setattr(_config, "DECIMAL_POINT", locale.localeconv()["decimal_point"])

        # convert
        result = xtoa(number)

        # restore saved locale
        locale.setlocale(locale.LC_NUMERIC, loc)

        assert result == expected_output

    def test_xtoa_works_correctly_for_opticstudio(self, simple_system):
        deltafocus = 0.5
        steps = 11
        # cfgfile
        fd, cfgoutfile = mkstemp(suffix=".CFG", prefix="zospy_")
        os.close(fd)

        # get analysis
        analysis = new_analysis(simple_system, constants.Analysis.AnalysisIDM.FftThroughFocusMtf)

        # Modify the settings file
        analysis_settings = analysis.GetSettings()
        analysis_settings.SaveTo(cfgoutfile)

        analysis_settings.ModifySettings(cfgoutfile, "TFM_DELTAFOC", xtoa(deltafocus, thousands_separator=None))
        analysis_settings.ModifySettings(cfgoutfile, "TFM_STEPS", xtoa(steps, thousands_separator=None))

        analysis_settings.LoadFrom(cfgoutfile)

        # Run analysis
        analysis.ApplyAndWaitForCompletion()

        os.remove(cfgoutfile)

        assert analysis.Settings.DeltaFocus == deltafocus
        assert analysis.Settings.NumberOfSteps == steps


class TestStringToNumberConversion:
    @pytest.mark.parametrize(
        "string,expected_output,dtype,decimal_point,thousands_separator",
        [
            ("1.5", 1.5, float, ".", ""),
            ("1,5", 1.5, float, ",", ""),
            ("1.5", 1.5, float, ".", ","),
            ("1,5", 1.5, float, ",", "."),
            ("1.5", 1.5, float, ".", None),
            ("1,5", 1.5, float, ",", None),
            ("2", 2, int, ".", ","),
            ("2", 2, int, ",", "."),
            ("1.234,5", 1234.5, float, ",", "."),
            ("1,234.5", 1234.5, float, ".", ","),
        ],
    )
    def test_atox_converts_string_correctly(self, string, expected_output, dtype, decimal_point, thousands_separator):
        result = atox(string, dtype=dtype, thousands_separator=thousands_separator, decimal_point=decimal_point)

        assert isinstance(result, dtype)
        assert result == expected_output

    @pytest.mark.parametrize(
        "string,dtype,thousands_separator,decimal_point",
        [
            ("1,5", float, "", "."),
            ("1.5", int, "", "."),
            ("1,234.5", float, "", "."),
        ],
    )
    def test_atox_raises_error_correctly(self, string, dtype, thousands_separator, decimal_point):
        with pytest.raises(ValueError):  # noqa: PT011
            atox(string, dtype=dtype, thousands_separator=thousands_separator, decimal_point=decimal_point)

    @pytest.mark.parametrize(
        "locale_cat,string,dtype,expected_output",
        [
            ("de_CH", "1’234.5", float, 1234.5),  # noqa: RUF001
            ("de_DE", "1.234,5", float, 1234.5),
            ("en_US", "1,234.5", float, 1234.5),
            ("nl_NL", "1.234,5", float, 1234.5),
            ("zh_CN", "1,234.5", float, 1234.5),
            ("de_CH", "1’234", int, 1234),  # noqa: RUF001
            ("de_DE", "1.234", int, 1234),
            ("en_US", "1,234", int, 1234),
            ("nl_NL", "1.234", int, 1234),
            ("zh_CN", "1,234", int, 1234),
        ],
    )
    def test_xtoa_converts_string_correctly_for_different_locale_categories(
        self, locale_cat, string, dtype, expected_output, monkeypatch
    ):
        if locale_cat not in locale.windows_locale.values():
            pytest.skip(f"Locale '{locale_cat}' not available.")

        # get new locale
        loc = locale.setlocale(locale.LC_NUMERIC)
        locale.setlocale(locale.LC_NUMERIC, locale_cat)

        monkeypatch.setattr(_config, "THOUSANDS_SEPARATOR", locale.localeconv()["thousands_sep"])
        monkeypatch.setattr(_config, "DECIMAL_POINT", locale.localeconv()["decimal_point"])

        # convert
        result = atox(string, dtype=dtype)

        # restore saved locale
        locale.setlocale(locale.LC_NUMERIC, loc)

        assert result == expected_output

    def test_atox_returns_correct_results_for_opticstudio(self, simple_system):
        xphase = 0.9
        surface = 3

        # cfgfile
        fd, cfgoutfile = mkstemp(suffix=".CFG", prefix="zospy_")
        os.close(fd)

        # txtfile
        fd, txtoutfile = mkstemp(suffix=".txt", prefix="zospy_")
        os.close(fd)

        analysis = new_analysis(simple_system, constants.Analysis.AnalysisIDM.PolarizationPupilMap)

        # Modify the settings file
        analysis_settings = analysis.GetSettings()
        analysis_settings.SaveTo(cfgoutfile)

        analysis_settings.ModifySettings(cfgoutfile, "PPM_PX", xtoa(xphase, thousands_separator=None))
        if isinstance(surface, str):
            analysis_settings.ModifySettings(cfgoutfile, "PPM_SURFACE", surface)
        else:
            analysis_settings.ModifySettings(cfgoutfile, "PPM_SURFACE", xtoa(surface, thousands_separator=None))

        analysis_settings.LoadFrom(cfgoutfile)

        # Run analysis
        analysis.ApplyAndWaitForCompletion()

        # Get results
        analysis.Results.GetTextFile(txtoutfile)

        with open(txtoutfile, encoding=simple_system.ZOS.get_txtfile_encoding()) as f:
            text_output = f.read()

        xphase_out = atox(_get_number_field("X-Phase", text_output), dtype=float)
        surface_out = atox(_get_number_field("Surface", text_output), dtype=int)

        os.remove(cfgoutfile)
        os.remove(txtoutfile)

        assert isinstance(xphase_out, float)
        assert xphase_out == xphase

        assert isinstance(surface_out, int)
        assert surface_out == surface



"""Utility functions for ZOSPy.

Mainly intended for internal use, `zospy.utils` contains utility functions for zospy. These functions are available
through its submodules:

- **`zospy.utils.clrutils`** provides utility functions for working with the ZOS-API .NET types;
- **`zospy.utils.pyutils`** provides utility functions for working with Python types;
- **`zospy.utils.zputils`** provides utility functions for working with Zemax OpticStudio.
"""

from zospy.utils import clrutils, pyutils, zputils

__all__ = ("zputils", "clrutils", "pyutils")



import clr
import pytest
from System.Reflection import Assembly

import zospy as zp
from zospy.api.codecs import OpticStudioInterfaceEncoder

# ruff: noqa: SLF001


class TestOpticStudioInterfaceEncoder:
    """Test automatic downcasting of generic interfaces to specific interfaces using `OpticStudioInterfaceEncoder`.

    Each test tests a specific scenario which fails is codecs are not used.
    """

    @pytest.fixture(scope="module")
    def zosapi_interfaces(self, zos):  # noqa: ARG002
        return Assembly.LoadFile(clr.FindAssembly("ZOSAPI_Interfaces"))

    def test_types_exist(self, zosapi_interfaces):
        """Test if all types in `OpticStudioInterfaceEncoder._interfaces` exist."""
        zosapi_type_names = [t.FullName for t in zosapi_interfaces.GetTypes()]

        assert OpticStudioInterfaceEncoder._interfaces.issubset(zosapi_type_names)

    def test_types_are_interfaces(self, zosapi_interfaces):
        """Test if all types in `OpticStudioInterfaceEncoder._interfaces` are interfaces."""
        assert all(zosapi_interfaces.GetType(t).IsInterface for t in OpticStudioInterfaceEncoder._interfaces)

    def test_register_interfaces_single(self, monkeypatch):
        # Temporarily overwrite OpticStudioInterfaceEncoder._interfaces and restore it after the test
        monkeypatch.setattr(OpticStudioInterfaceEncoder, "_interfaces", frozenset())

        interface = "ZOSAPI.Editors.NCE.IObjectScatteringSettings"

        assert interface not in OpticStudioInterfaceEncoder._interfaces

        OpticStudioInterfaceEncoder.register_interfaces(interface)

        assert interface in OpticStudioInterfaceEncoder._interfaces

    def test_register_interfaces_multiple(self, monkeypatch):
        # Temporarily overwrite OpticStudioInterfaceEncoder._interfaces and restore it after the test
        monkeypatch.setattr(OpticStudioInterfaceEncoder, "_interfaces", frozenset())

        interfaces = ["ZOSAPI.Editors.NCE.IObjectScatteringSettings", "ZOSAPI.Editors.NCE.IVolumePhysicsModelSettings"]

        assert all(i not in OpticStudioInterfaceEncoder._interfaces for i in interfaces)

        OpticStudioInterfaceEncoder.register_interfaces(interfaces)

        assert all(i in OpticStudioInterfaceEncoder._interfaces for i in interfaces)

    def test_analysis_settings_ias(self, oss):
        """Test downcasting for `ZOSAPI.Analysis.Settings.IAS_`."""
        analysis = oss.Analyses.New_Analysis_SettingsFirst(
            zp.constants.Analysis.AnalysisIDM.ZernikeStandardCoefficients
        )

        analysis.GetSettings().MaximumNumberOfTerms = 37

        assert analysis.GetSettings().MaximumNumberOfTerms == 37

    def test_editors_nce_iobject(self, nsc_simple_system):
        """Test downcasting for `ZOSAPI.Editors.NCE.IObject`."""
        object_data = nsc_simple_system.NCE.GetObjectAt(2).ObjectData

        assert object_data.Radius1 == 20

    def test_editors_lde_isurface(self, polarized_system):
        """Test downcasting for `ZOSAPI.Editors.LDE.ISurface`."""
        surface_data = polarized_system.LDE.GetSurfaceAt(2).SurfaceData

        assert surface_data.Ar == 1
        assert surface_data.Dr == 0

    def test_editors_lde_isurfaceaperturetype(self, simple_system):
        """Test downcasting for `ZOSAPI.Editors.LDE.ISurfaceApertureType`."""
        stop_surface = simple_system.LDE.GetSurfaceAt(simple_system.LDE.StopSurface)

        spider_aperture = stop_surface.ApertureData.CreateApertureTypeSettings(
            zp.constants.Editors.LDE.SurfaceApertureTypes.Spider
        )
        spider_aperture.NumberOfArms = 10
        stop_surface.ApertureData.ChangeApertureTypeSettings(spider_aperture)

        assert stop_surface.ApertureData.CurrentTypeSettings.NumberOfArms == 10

    def test_editors_lde_isurfacescatteringtype(self, simple_system):
        """Test downcasting for `ZOSAPI.Editors.LDE.ISurfaceScatteringType`."""
        lens_front = simple_system.LDE.GetSurfaceAt(2)

        gaussian_scattering = lens_front.ScatteringData.CreateScatteringTypeSettings(
            zp.constants.Editors.LDE.SurfaceScatteringTypes.Gaussian
        )
        gaussian_scattering.ScatterFraction = 0.5
        lens_front.ScatteringData.ChangeScatteringTypeSettings(gaussian_scattering)

        assert lens_front.ScatteringData.CurrentTypeSettings.ScatterFraction == 0.5

    def test_tools_isystemtool(self, oss):
        """Test downcasting for `ZOSAPI.Tools.ISystemTool`."""
        local_optimization = oss.Tools.OpenLocalOptimization()
        local_optimization.NumberOfCores = 5

        assert oss.Tools.CurrentTool.NumberOfCores == 5



from types import SimpleNamespace

from zospy.utils.zputils import unpack_datagrid

mock_datagrid = SimpleNamespace(
    Values=[[1, 2, 3], [4, 5, 6], [7, 8, 9]],
    MinX=-1.5,
    MinY=-1.5,
    Nx=3,
    Ny=3,
    Dx=1,
    Dy=1,
    XLabel="x",
    YLabel="y",
)


def test_unpack_datagrid_bottom_left():
    result = unpack_datagrid(mock_datagrid, cell_origin="bottom_left")

    assert all(result.columns == [-1, 0, 1])
    assert all(result.index == [-1, 0, 1])


def test_unpack_datagrid_center():
    result = unpack_datagrid(mock_datagrid, cell_origin="center")

    assert all(result.columns == [-1.5, -0.5, 0.5])
    assert all(result.index == [-1.5, -0.5, 0.5])



"""Base functionality for analyses."""

from __future__ import annotations

import copy
import json
from collections.abc import MutableMapping
from dataclasses import asdict, dataclass, is_dataclass
from datetime import datetime
from enum import Enum
from importlib import import_module
from types import SimpleNamespace
from typing import Any, Literal

import numpy as np
import pandas as pd

from zospy.api import _ZOSAPI, constants
from zospy.utils.clrutils import system_datetime_to_datetime
from zospy.zpcore import OpticStudioSystem


def _pprint(d, indent=0):
    """Pretty print an attribute dict."""
    items = []
    for key, value in sorted(d.items(), key=lambda x: str(x[0])):
        if isinstance(key, str):
            strkey = f"'{key}'"
        else:
            strkey = str(key)
        if isinstance(value, MutableMapping):
            items.append(" " * indent + strkey + ":")
            items.extend(_pprint(value, indent + 2))
        else:
            items.append(" " * indent + strkey + ": " + repr(value))
    return items


def _convert_dicttype(dictionary, newtype=dict, convert_nested=True, method="deepcopy"):
    """Packs a nested dictionary to a one-level dictionary with tuple keys.

    Parameters
    ----------
    dictionary : dict
        A dictionary or dictionary subtype, optionally with nested dictionaries
    newtype : type
        The new dictionary (sub)type
    convert_nested : bool
        Whether nested dictionaries should be converted as well. Defaults to True
    method : str
        The method used to copy values. One of 'deepcopy', 'copy' and 'assign'. Defaults to 'deepcopy'

    Returns
    -------
    object
        The converted dictionary type
    """
    ret = newtype()
    for key, value in dictionary.items():
        if convert_nested and isinstance(value, dict):
            ret[key] = _convert_dicttype(value, newtype=newtype, convert_nested=convert_nested, method=method)
        else:
            if method == "deepcopy":
                ret[key] = copy.deepcopy(value)
            elif method == "copy":
                ret[key] = copy.copy(value)
            elif method == "assign":
                ret[key] = value
            else:
                raise ValueError()
    return ret


class _AnalysisResultJSONEncoder(json.JSONEncoder):
    """Serializes an `AnalysisResult` object to JSON."""

    def __init__(self, sort_keys=False, indent=4, **kwargs):
        super().__init__(sort_keys=sort_keys, indent=indent, **kwargs)

    def default(self, o: Any) -> Any:
        if isinstance(o, pd.DataFrame):
            return {"__type__": self._type_name(o), "data": o.to_dict(orient="tight")}

        if isinstance(o, pd.Series):
            if isinstance(o.index, pd.MultiIndex):
                return {
                    "__type__": self._type_name(o),
                    "data": o.to_frame().to_dict(orient="tight"),
                    "__multi_index__": True,
                }

            return {"__type__": self._type_name(o), "data": o.to_dict()}

        if isinstance(o, np.ndarray):
            return {"__type__": self._type_name(o), "data": o.tolist()}

        if is_dataclass(o):
            return {
                "__type__": "dataclass",
                "__type_name__": self._type_name(o),
                "__type_module__": type(o).__module__,
                "data": asdict(o),
            }

        if isinstance(o, datetime):
            return {"__type__": self._type_name(o), "data": o.isoformat()}

        return super().default(o)

    def encode(self, o: Any) -> str:
        if isinstance(o, AttrDict):
            o = self._process_attrdict(o)

        return super().encode(o)

    def _process_attrdict(self, o: AttrDict) -> dict:
        object_type = self._type_name(o)

        if any(isinstance(v, AttrDict) for v in o.values()):
            o = {k: (self._process_attrdict(v) if isinstance(v, AttrDict) else v) for k, v in o.items()}

        return {"__type__": object_type, "data": dict(o)}

    @staticmethod
    def _type_name(o: Any) -> str:
        return type(o).__name__


class _AnalysisResultJSONDecoder(json.JSONDecoder):
    """Reconstructs an `AnalysisResult` object from its JSON representation."""

    def __init__(self):
        super().__init__(object_hook=self.object_hook)

    @staticmethod
    def object_hook(o: dict) -> Any:
        if object_type := o.get("__type__"):
            if object_type == "DataFrame":
                return pd.DataFrame.from_dict(o["data"], orient="tight")

            if object_type == "Series":
                if "__multi_index__" in o:
                    return pd.DataFrame.from_dict(o["data"], orient="tight").iloc[:, 0]

                return pd.Series(o["data"])

            if object_type == "ndarray":
                return np.array(o["data"])

            if object_type == "dataclass":
                return _AnalysisResultJSONDecoder._process_dataclass(o)

            if object_type == "AttrDict":
                return AttrDict.from_dict(o["data"])

            if object_type == "datetime":
                return datetime.fromisoformat(o["data"])

            return o["data"]

        return o

    @staticmethod
    def _process_dataclass(o: dict) -> Any:
        module = o["__type_module__"]
        name = o["__type_name__"]

        if module.startswith("zospy.analyses"):
            try:
                m = import_module(module)
                return getattr(m, name)(**o["data"])
            except (ModuleNotFoundError, AttributeError):
                return SimpleNamespace(**o["data"])

        return SimpleNamespace(**o["data"])


@dataclass(frozen=True)
class AnalysisMessage:
    """Zemax OpticStudio analysis message."""

    ErrorCode: str
    Message: str


@dataclass(frozen=True)
class AnalysisMetadata:
    """Zemax OpticStudio analysis metadata."""

    DateTime: datetime
    FeatureDescription: str
    LensFile: str
    LensTitle: str


class AttrDict(dict):
    """Basically a dict with attribute access.

    Equal to scipy's OptimizeResult (https://github.com/scipy/scipy/blob/v1.6.3/scipy/optimize/optimize.py#L82-L138).
    """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            return "\n".join(_pprint(self))
        else:
            return self.__class__.__name__ + "()"

    def __dir__(self):
        return list([key for key in self.keys() if str(key).isidentifier()]) + dir(dict)

    def to_dict(self, convert_nested=True, method="deepcopy"):
        """Convert the AttrDict to a standard dict.

        Parameters
        ----------
        convert_nested : bool
            Whether nested dictionaries should be converted as well. Defaults to True
        method : str
            The method used to copy values. One of 'deepcopy', 'copy' and 'assign'. Defaults to 'deepcopy'

        Returns
        -------
        dict
            The AttrDict as standard Dict
        """
        return _convert_dicttype(self, newtype=dict, convert_nested=convert_nested, method=method)

    @classmethod
    def from_dict(cls, dictionary, convert_nested=True, method="deepcopy"):
        """Create an AttrDict from a standard dict.

        Parameters
        ----------
        dictionary : dict
            The dictionary that is to be converted
        convert_nested : bool
            Whether nested dictionaries should be converted as well. Defaults to True
        method : str
            The method used to copy values. One of 'deepcopy', 'copy' and 'assign'. Defaults to 'deepcopy'

        Returns
        -------
        AttrDict
            The converted AttrDict
        """
        return _convert_dicttype(dictionary, newtype=cls, convert_nested=convert_nested, method=method)


class AnalysisResult(AttrDict):
    def __init__(
        self,
        analysistype: str,
        data: Any = None,
        settings: pd.Series = None,
        metadata: AnalysisMetadata = None,
        headerdata: list[str] = None,
        messages: list[AnalysisMessage] = None,
        **kwargs,
    ):
        """A class designed to hold an OpticStudio analysis.

        The class is basically a dict with attribute level access. However, for an AnalysisResult, certain keys are
        often present and thus automatically set.

        Parameters
        ----------
        analysistype : str
            The type of analysis that has been performed. Will be assigned to AnalysisResult.AnalysisType (also
            available through AnalysisResult['AnalysisType'].
        data : Any, optional
            The analysis data, can be any of the native python datatypes, a pd.Series, pd.DataFrame, np.ndarray or
            AnalysisData. Will be assigned to AnalysisResult.Data (also available through AnalysisResult['Data'].
            Defaults to None.
        settings : pd.Series, optional
            The analysis settings. Will be assigned to AnalysisResult.Settings (also available through
            AnalysisResult['Settings']. Defaults to None.
        metadata : AnalysisMetadata, optional
            The analysis metadata. Will be assigned to AnalysisResult.MetaData (also available through
            AnalysisResult['MetaData']. Defaults to None.
        headerdata : list[str], optional
            The analysis headerdata. Will be assigned to AnalysisResult.HeaderData (also available through
            AnalysisResult['HeaderData']. Defaults to None.
        messages : list[AnalysisMessage], optional:
            The analysis messages. Will be assigned to AnalysisResult.Messages (also available through
            AnalysisResult['Messages']. Defaults to None.
        kwargs:
            Any supplied kwarg will be assigned as an attribute (also available as AnalysisResult[kwarg]. Note that
            'AnalysisType', 'Data', 'Settings' 'MetaData', 'HeaderData' and 'Messages' are not available to be set.

        Returns
        -------
        AnalysisResult:
            A dict with attribute-like access

        Raises
        ------
        SyntaxError:
            If any of the following kwargs is specified 'AnalysisType', 'Data', 'Settings' 'MetaData', 'HeaderData' or
            'Messages'. These are to be set through the default arguments.
        """
        super().__init__(
            AnalysisType=analysistype,
            Data=data,
            Settings=settings,
            MetaData=metadata,
            HeaderData=headerdata,
            Messages=messages,
            **kwargs,
        )

    def to_json(self) -> str:
        """Serializes the analysis result to JSON.

        Returns
        -------
        json_string : str
            A JSON representation of the analysis result.
        """
        return _AnalysisResultJSONEncoder().encode(self)

    @classmethod
    def from_json(cls, json_string: str) -> AnalysisResult:
        """Creates an `AnalysisResult` object from a JSON string.

        This method should only be used with JSON strings created with `AnalysisResult.to_json`.

        Known limitations
        =================

        - Pandas `Series` with a numeric index are not compatible with JSON, which only allows string keys. The
        reconstructed value will therefore not equal the original value. Use string indices Numpy arrays where possible.
        - Support for custom attributes is limited and requires

        Parameters
        ----------
        json_string : str
            JSON representation of an `AnalysisResult` object, created with `to_json`.

        Returns
        -------
        result : AnalysisResult
            The reconstructed `AnalysisResult`.
        """
        parsed_json = _AnalysisResultJSONDecoder().decode(json_string)

        result = cls(analysistype="")
        result.update(parsed_json)

        return result


class OnComplete(str, Enum):
    """Action to perform after running an OpticStudio analysis.

    This `Enum` can be passed to analysis methods in `zospy.analyses`.

    Attributes
    ----------
    Close : str
        Close the analysis.
    Release : str
        Keep the analysis open, but not active.
    Sustain : str
        Keep the analysis open and active.

    Examples
    --------
    Run a System Data analysis and keep the analysis open:

    >>> from zospy.analyses import OnComplete
    >>> from zospy.analyses.reports import system_data
    >>> result = system_data(oss, OnComplete.Sustain)
    """

    Close = "Close"
    """Close the analysis."""

    Release = "Release"
    """Keep the analysis open, but not active."""

    Sustain = "Sustain"
    """Keep the analysis open and active."""


class Analysis:
    def __init__(self, analysis: _ZOSAPI.Analysis.IA_):
        """Zemax OpticStudio Analysis with full access to its Settings and Results attributes.

        Parameters
        ----------
        analysis : ZOSAPI.Analysis.IA_
            analysis object
        """
        self._analysis = analysis

    @property
    def Settings(self) -> _ZOSAPI.Analysis.Settings.IAS_:
        """Analysis-specific settings."""
        return self._analysis.GetSettings()

    @property
    def Results(self) -> _ZOSAPI.Analysis.Data.IAR_:
        """Analysis results."""
        return self._analysis.GetResults()

    def complete(self, oncomplete: OnComplete | str, result: AnalysisResult) -> AnalysisResult:
        """Completes the analysis by either closing, releasing or sustaining it.

        Parameters
        ----------
        oncomplete : OnComplete | str
            Action to perform on completion.
        result : AnalysisResult
            Result of the analysis.

        Returns
        -------
        `result` if the analysis is closed or released, `result` extended with the `Analysis` object if the analysis
        is sustained.
        """
        oncomplete = OnComplete(oncomplete)

        if oncomplete == OnComplete.Close:
            self._analysis.Close()
            return result

        if oncomplete == OnComplete.Release:
            self._analysis.Release()
            return result

        if oncomplete == OnComplete.Sustain:
            result.Analysis = self
            return result

        raise ValueError(f"oncomplete should be a member of zospy.analyses.base.OnComplete, got {oncomplete}")

    def set_field(self, value: int | str):
        """Sets the field value for the analysis.

        Parameters
        ----------
        value : int or str
            The value to which the field should be set. Either int or str. Accepts only 'All' as string.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            When 'value' is not integer or string. When it is a string, it also raises an error when the string
            does not
            equal 'All'.
        """
        if value == "All":
            self.Settings.Field.UseAllFields()
        elif isinstance(value, int):
            self.Settings.Field.SetFieldNumber(value)
        else:
            raise ValueError(f'Field value should be "All" or an integer, got {value}')

    def get_field(self) -> int | str:
        """Gets the wavelength value of the analysis.

        Returns
        -------
        int | str
            Either the field number, or 'All' if field was set to 'All'.
        """
        field_number = self.Settings.Field.GetFieldNumber()

        return "All" if field_number == 0 else field_number

    def get_header_data(self) -> list[str]:
        """Obtains the header data from an OpticStudio analysis.

        Returns
        -------
        list
            The header data.
        """
        return list(self.Results.HeaderData.Lines)

    def get_messages(self) -> list[AnalysisMessage]:
        """Obtains the messages from the analysis.

        Returns
        -------
        list[AnalysisMessage]
            A list of messages. Each item is a NamedTuple with names (ErrorCode, Message).
        """
        result = []

        for i in range(self.Results.NumberOfMessages):
            message = self.Results.GetMessageAt(i)
            error_code = str(message.ErrorCode)

            result.append(AnalysisMessage(error_code, message.Text))

        return result

    def get_metadata(self) -> AnalysisMetadata:
        """Obtains the metadata from the analysis.

        Returns
        -------
        AnalysisMetadata
            A named tuple containing the MetaData including 'DateTime', 'FeatureDescription', 'LensFile'
            and 'LensTitle'.
        """
        result = AnalysisMetadata(
            system_datetime_to_datetime(self.Results.MetaData.Date),
            self.Results.MetaData.FeatureDescription,
            self.Results.MetaData.LensFile,
            self.Results.MetaData.LensTitle,
        )

        return result

    def set_wavelength(self, value: int | str):
        """Sets the wavelength value for the analysis.

        Parameters
        ----------
        value : int | str
            The value to which the wavelength should be set. Either int or str. Accepts only 'All' as string.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            When 'value' is not integer or string. When it is a string, it also raises an error when the string does not
            equal 'All'.
        """
        if value == "All":
            self.Settings.Wavelength.UseAllWavelengths()
        elif isinstance(value, int):
            self.Settings.Wavelength.SetWavelengthNumber(value)
        else:
            raise ValueError('Wavelength value should be "All" or an integer')

    def get_wavelength(self) -> int | str:
        """Gets the wavelength value of the analysis.

        Returns
        -------
        int | str
            Either the wavelength number, or 'All' if wavelength was set to 'All'.
        """
        wavelength = self.Settings.Wavelength.GetWavelengthNumber()

        return "All" if wavelength == 0 else wavelength

    def set_surface(self, value: int | Literal["Image", "Objective", "Object"]):
        """Sets the surface value for the analysis.

        Parameters
        ----------
        value : int or Literal["Image", "Objective", "Object"]
            The value to which the surface should be set. Either int or str. Accepts only 'Image' or 'Objective' as
            string.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            When 'value' is not integer or string. When it is a string, it also raises an error when the string does not
            equal 'Image', 'Objective' or 'Object'.
        """
        if value == "Image":
            self.Settings.Surface.UseImageSurface()
        elif value in ("Object", "Objective"):
            self.Settings.Surface.UseObjectiveSurface()
        elif isinstance(value, int):
            self.Settings.Surface.SetSurfaceNumber(value)
        else:
            raise ValueError(f'Surface value should be "Image", "Objective" or an integer, got {value}')

    def __getattr__(self, item):
        return getattr(self._analysis, item)

    def __dir__(self):
        return sorted(set(super().__dir__()).union(dir(self._analysis)))


def new_analysis(
    oss: OpticStudioSystem, analysis_type: constants.Analysis.AnalysisIDM, settings_first: bool = True
) -> Analysis:
    """Creates a new analysis in Zemax.

    Parameters
    ----------
    oss : OpticStudioSystem
        The Zemax OpticStudio system
    analysis_type : zospy.constants.Analysis.AnalysisIDM
        Analysis type from `ZOSAPI.Analysis.AnalysisIDM`
    settings_first : bool
        Do not run the analysis immediately, which allows to adjust settings. Defaults to `True`.

    Returns
    -------
    analysis : Analysis

    Examples
    --------
    >>> new_analysis(oss, zp.constants.Analysis.AnalysisIDM.ZernikeStandardCoefficients)
    """
    analysis = (
        oss.Analyses.New_Analysis_SettingsFirst(analysis_type)
        if settings_first
        else oss.Analyses.New_Analysis(analysis_type)
    )
    return Analysis(analysis)



from __future__ import annotations

import os
import subprocess
from pathlib import Path
from typing import Any

EXECUTOR_PYTHON_VERSION: str = "3.12"
SCRIPT_DIR = Path(__file__).parent.resolve(strict=True)


def _dependencies_args(dependencies: set[str]) -> list[str]:
    result = []

    for dep in dependencies:
        result.extend(["--with", dep])

    return result


def _quote(s: Any) -> str:
    return f'"{s!s}"'


def get_dependencies_from_requirements(requirements: Path) -> list[str]:
    with requirements.open("r") as f:
        return [line.strip() for line in f.readlines() if line.strip()]


def run_example_with_uv(
    example_path: Path,
    output_path: Path | None = None,
    requirements: Path | None = None,
    *,
    editable_zospy_location: Path | None = None,
    keep_extension_mode: bool = False,
) -> bool:
    # Build dependencies string
    dependencies = {"ipykernel"}

    if requirements is not None:
        dependencies.update(get_dependencies_from_requirements(requirements))

    # Use the editable zospy location if provided, otherwise use the required zospy version
    required_zospy = next((dep for dep in dependencies if dep.startswith("zospy")), None)
    dependencies.discard(required_zospy)
    dependencies_arguments = _dependencies_args(dependencies)

    if editable_zospy_location is not None:
        dependencies_arguments.extend(["--with-editable", editable_zospy_location.as_posix()])
    elif required_zospy is not None:
        dependencies_arguments.extend(["--with", required_zospy])
    else:
        dependencies_arguments.extend(["--with", "zospy"])

    parameters = []

    if output_path is not None:
        parameters.extend(["--output", str(output_path)])

    if keep_extension_mode:
        parameters.append("--keep-extension-mode")

    # Use `uv` supplied by Hatch when running in a Hatch environment
    uv_exe = os.environ.get("HATCH_UV", "uv")

    command = [
        uv_exe,
        "run",
        "--python",
        EXECUTOR_PYTHON_VERSION,
        *dependencies_arguments,
        "--no-project",  # Do not run in the project environment
        str(SCRIPT_DIR / "run_example.py"),
        ".",  # The command is run in the example directory
        *parameters,
    ]

    print(f"Running example {example_path} with command: {' '.join(command)}")

    return subprocess.run(command, check=False, cwd=example_path).returncode == 0


def process_example(
    example_path: Path,
    output_path: Path | None = None,
    editable_zospy_location: str | None = None,
    *,
    keep_extension_mode: bool = False,
) -> bool:
    """Run a single example and save the output."""
    requirements = example_path / "requirements.txt" if (example_path / "requirements.txt").exists() else None

    # Run the example
    return run_example_with_uv(
        example_path,
        output_path,
        requirements,
        editable_zospy_location=editable_zospy_location,
        keep_extension_mode=keep_extension_mode,
    )



import locale
from pathlib import Path
from sys import version_info
from types import SimpleNamespace

import pytest

import zospy as zp
from zospy import constants

# ruff: noqa: SLF001


def patch_zos(zos: zp.ZOS, monkeypatch: pytest.MonkeyPatch):
    patch_application = SimpleNamespace(IsValidLicenseForAPI=False)
    patch_connection = SimpleNamespace(
        IsAlive=False,
        ConnectAsExtension=lambda n: patch_application,  # noqa: ARG005
        CreateNewApplication=lambda: patch_application,
    )

    def patch_assign_connection(self: zp.ZOS):
        self.Connection = patch_connection

    # Patch ZOS class so ZOS.Application.IsValidLicenseForAPI is False
    monkeypatch.setattr(zos, "_assign_connection", patch_assign_connection.__get__(zos, zp.ZOS))
    # All attributes need to be patched explicitly, otherwise they won't be restored
    monkeypatch.setattr(zos, "Application", patch_application)
    monkeypatch.setattr(zos, "Connection", patch_connection)


def test_connect_without_valid_license_raises_exception(zos, connection_mode, monkeypatch):
    patch_zos(zos, monkeypatch)

    with pytest.raises(ConnectionRefusedError):
        zos.connect(connection_mode)


def test_load_zos_dlls_with_nethelper_and_opticstudio_directory_raises_valueerror(zos):
    with pytest.raises(ValueError, match="Only one of `zosapi_nethelper` and `opticstudio_directory` may be specified"):
        zos._load_zos_dlls(zosapi_nethelper="dummy/path", opticstudio_directory="dummy/path")


@pytest.mark.must_pass  # Other tests will fail if this one does
def test_zos_singleton(zos, oss):  # noqa: ARG001
    assert zos.Application is not None

    with pytest.warns(match=r"Only a single instance of ZOS can exist at any time\. Returning existing instance\."):
        zos2 = zp.ZOS()

        # If init is called again, the Application attribute will be set to None
        assert zos2.Application is not None
        assert zos2 is zos


@pytest.mark.filterwarnings("ignore:Only a single instance of ZOS can exist at any time")
def test_zos_singleton_logs(zos, caplog):
    with caplog.at_level("DEBUG"):
        zos2 = zp.ZOS()

    assert "ZOS instance already initialized" in caplog.text
    assert "Initializing ZOS instance" not in caplog.text
    assert zos2 is zos


def test_zos_get_instance(zos):
    assert zp.ZOS.get_instance() is zos


@pytest.mark.must_pass
def test_can_connect(oss):
    assert oss._System is not None


def test_can_disconnect(zos, oss):  # noqa: ARG001
    assert zos.Application is not None  # The Application object should be available

    zos.disconnect()

    assert zos.Application is None


@pytest.mark.require_mode("extension")
def test_get_primary_system_populates_openfile(zos, oss):  # noqa: ARG001
    assert oss._OpenFile == oss.SystemFile


@pytest.mark.must_pass
def test_create_simple_system(simple_system):
    assert simple_system.LDE.NumberOfSurfaces == 5


def test_version(optic_studio_version):
    assert optic_studio_version


def test_new(simple_system):
    # Replace with a new system
    simple_system.new()

    assert simple_system.LDE.NumberOfSurfaces == 3


XFAIL_ZOS_REASON = "The .zos file format is only supported by OpticStudio 21.3 and higher"


@pytest.mark.parametrize(
    "filename",
    [
        "simple_system.zmx",
        pytest.param(
            "simple_system.zos", marks=pytest.mark.xfail_for_opticstudio_versions(["<21.3.0"], XFAIL_ZOS_REASON)
        ),
    ],
)
class TestLoad:
    @pytest.fixture
    def demo_systems_folder(self, request) -> Path:
        return request.config.rootpath / "tests/data/optical_systems"

    def test_load(self, oss, demo_systems_folder, filename):
        path = (demo_systems_folder / filename).resolve()

        oss.load(path)

        assert oss.LDE.NumberOfSurfaces == 5
        assert oss.LDE.GetSurfaceAt(2).Radius == 20.0

    def test_load_relative(self, oss, demo_systems_folder, filename, monkeypatch):
        monkeypatch.chdir(demo_systems_folder)

        oss.load(filename)

        assert oss.LDE.NumberOfSurfaces == 5
        assert oss.LDE.GetSurfaceAt(2).Radius == 20.0


@pytest.mark.parametrize(
    "filename",
    [
        "TEST.ZMX",
        pytest.param("TEST.ZOS", marks=pytest.mark.xfail_for_opticstudio_versions(["<21.3.0"], XFAIL_ZOS_REASON)),
    ],
)
class TestSaveAs:
    def test_save_as(self, simple_system, tmp_path, filename):
        save_path = tmp_path / filename

        simple_system.save_as(save_path.resolve())

        assert save_path.exists()

    def test_save_as_relative_path(self, simple_system, tmp_path, filename, monkeypatch):
        monkeypatch.chdir(tmp_path)

        save_path = tmp_path / filename

        simple_system.save_as(filename)

        assert save_path.exists()


def test_get_system(zos, oss, connection_mode):
    if connection_mode == "extension":
        pytest.xfail(reason="GetSystem does not work correctly in extension mode due to a bug in the ZOS-API.")

    system = zos.get_system(0)

    assert system._System is not None
    assert system.SystemID == oss.SystemID


@pytest.mark.require_mode("standalone")
def test_create_new_system(zos, oss, connection_mode):  # noqa: ARG001
    new_system = zos.create_new_system()

    assert zos.Application.NumberOfOpticalSystems == 2
    assert oss.SystemID != new_system.SystemID


@pytest.mark.require_mode("extension")
def test_create_new_system_raises_valueerror(zos, simple_system, connection_mode):  # noqa: ARG001
    with pytest.raises(ValueError, match="Can only create a new system when using a standalone connection"):
        zos.create_new_system()


@pytest.fixture
def oss_with_modifiable_config(zos: zp.ZOS, connection_mode, tmp_path) -> zp.zpcore.OpticStudioSystem:
    config_file = tmp_path / "opticstudio_config_file.CFG"
    zos.Connection.PreferencesFile = str(config_file)

    oss = zos.connect(connection_mode)

    yield oss

    # Close the system
    if oss is not None:
        zos.Application.CloseApplication()


@pytest.mark.require_mode("standalone")
class TestTxtFileEncoding:
    @pytest.mark.parametrize(
        "txtfile_encoding,expected_encoding", [("Unicode", "UTF-16-le"), ("ANSI", "LocalePreferredEncoding")]
    )
    def test_get_txtfile_encoding_returns_correct_result(
        self, oss_with_modifiable_config, txtfile_encoding, expected_encoding, monkeypatch: pytest.MonkeyPatch
    ):
        def getencoding(*args, **kwargs):  # noqa: ARG001
            return "LocalePreferredEncoding"

        if version_info < (3, 11):
            monkeypatch.setattr(locale, "getpreferredencoding", getencoding)
        else:
            monkeypatch.setattr(locale, "getencoding", getencoding)

        oss_with_modifiable_config.ZOS.Application.Preferences.General.TXTFileEncoding = getattr(
            constants.Preferences.EncodingType, txtfile_encoding
        )

        returned_encoding = oss_with_modifiable_config.ZOS.get_txtfile_encoding()

        assert returned_encoding == expected_encoding

    @pytest.mark.parametrize("txtfile_encoding", ["Unicode", "ANSI"])
    def test_analysis_result_parsed_with_correct_encoding(self, oss_with_modifiable_config, txtfile_encoding, tmp_path):
        oss_with_modifiable_config.ZOS.Application.Preferences.General.TXTFileEncoding = getattr(
            constants.Preferences.EncodingType, txtfile_encoding
        )

        analysis = zp.analyses.base.new_analysis(
            oss_with_modifiable_config, zp.constants.Analysis.AnalysisIDM.SystemData
        )
        analysis.ApplyAndWaitForCompletion()
        txtoutfile = str(tmp_path / "test_analysis_result_parsed_with_correct_encoding.txt")
        analysis.Results.GetTextFile(txtoutfile)

        with open(txtoutfile, encoding=oss_with_modifiable_config.ZOS.get_txtfile_encoding()) as txtfile:
            assert "System/Prescription Data" in txtfile.readline()



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("BackgroundColors", "DateTimeType", "EncodingType", "LanguageType", "ShowLineAsType", "ZemaxThemes")

class BackgroundColors:
    Default: BackgroundColors = None
    White: BackgroundColors = None
    LightGray: BackgroundColors = None
    Gray: BackgroundColors = None
    DarkGray: BackgroundColors = None
    Black: BackgroundColors = None

class DateTimeType:
    None_: DateTimeType = None
    Date: DateTimeType = None
    DateTime: DateTimeType = None

class EncodingType:
    ANSI: EncodingType = None
    Unicode: EncodingType = None

class LanguageType:
    Chinese: LanguageType = None
    English: LanguageType = None
    Japanese: LanguageType = None

class ShowLineAsType:
    TextAbove: ShowLineAsType = None
    FileName: ShowLineAsType = None
    ConfigurationNumber: ShowLineAsType = None

class ZemaxThemes:
    Classic: ZemaxThemes = None
    Dark: ZemaxThemes = None



"""Zemax OpticStudio analyses from the MTF category."""

from __future__ import annotations

import os
from tempfile import mkstemp

import pandas as pd

from zospy import utils
from zospy.analyses.old.base import AnalysisResult, OnComplete, new_analysis
from zospy.api import constants
from zospy.zpcore import OpticStudioSystem


def _correct_fft_through_focus_mtftype_api_bug(oss, analysis, mtftype) -> None:
    """Correction for an API bug in OpticStudio versions < 21.2.

    In these versions, the MTF Type cannot be set through the ZOS-API for the FFT Through Focus MTF
    See also: https://community.zemax.com/zos-api-12/zos-api-setting-mtf-property-type-not-working-730

    Parameters
    ----------
    oss : zospy.zpcore.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    analysis : Analysis
        An FFT Through Focus MTF analysis.
    mtftype : zospy.constants.Analysis.Settings.Mtf.MtfTypes.Modulation

    Returns
    -------
    None
    """
    if oss.ZOS.version < "21.2.0":
        fd, cfgoutfile = mkstemp(suffix=".CFG", prefix="zospy_")
        os.close(fd)
        analysis.Settings.SaveTo(cfgoutfile)

        analysis.Settings.ModifySettings(
            cfgoutfile,
            "TFM_TYPE",
            str(int(constants.process_constant(constants.Analysis.Settings.Mtf.MtfTypes, mtftype))),
        )
        analysis.Settings.LoadFrom(cfgoutfile)
        os.remove(cfgoutfile)


def fft_through_focus_mtf(
    oss: OpticStudioSystem,
    sampling: str | int = "64x64",
    deltafocus: float = 0.1,
    frequency: float = 0,
    numberofsteps: int = 5,
    wavelength: str | int = "All",
    field: str | int = "All",
    mtftype: constants.Analysis.Settings.Mtf.MtfTypes | str = "Modulation",
    use_polarization: bool = False,
    use_dashes: bool = False,
    oncomplete: OnComplete | str = OnComplete.Close,
) -> AnalysisResult:
    """Wrapper around the OpticStudio FFT Through Focus MTF.

    For an in depth explanation of the parameters, see the Zemax OpticStudio user manual

    Parameters
    ----------
    oss : zospy.zpcore.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    sampling : str | int
        The sampling, either string (e.g. '64x64') or int. The integer will be treated as a ZOSAPI Constants integer.
    deltafocus : float
        The delta focus, defaults to 0.1
    frequency : float
        The frequency. Defaults to 0.
    numberofsteps : int
        The number of steps. Defaults to 5.
    wavelength : str | int
        The wavelength to use in the MTF. Either 'All' or an integer specifying the wavelength number.
    field : str | int
        The field to use in the MTF. Either 'All' or an integer specifying the field number.
    mtftype : zospy.constants.Analysis.Settings.Mtf.MtfTypes.Modulation
        The MTF type (e.g. `Modulation`) that is calculated.
    use_polarization : bool
        Use polarization. Defaults to False.
    use_dashes : bool
        Use dashes. Defaults to False.
    oncomplete : OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.

    Returns
    -------
    AnalysisResult
        A FftThroughFocusMtf analysis result
    """
    analysis_type = constants.Analysis.AnalysisIDM.FftThroughFocusMtf

    # Create analysis
    analysis = new_analysis(oss, analysis_type)

    # Apply settings
    analysis.Settings.SampleSize = getattr(constants.Analysis.SampleSizes, utils.zputils.standardize_sampling(sampling))
    analysis.Settings.DeltaFocus = deltafocus
    analysis.Settings.Frequency = frequency
    analysis.Settings.NumberOfSteps = numberofsteps
    analysis.set_wavelength(wavelength)
    analysis.set_field(field)
    analysis.Settings.Type = constants.process_constant(constants.Analysis.Settings.Mtf.MtfTypes, mtftype)
    analysis.Settings.UsePolarization = use_polarization
    analysis.Settings.UseDashes = use_dashes

    # Correct an API bug in setting API type for OpticStudio version <21.2
    _correct_fft_through_focus_mtftype_api_bug(oss, analysis, mtftype)

    # Calculate
    analysis.ApplyAndWaitForCompletion()

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Get settings
    settings = pd.Series(name="Settings", dtype=object)

    settings.loc["SampleSize"] = str(analysis.Settings.SampleSize)
    settings.loc["DeltaFocus"] = analysis.Settings.DeltaFocus
    settings.loc["Frequency"] = analysis.Settings.Frequency
    settings.loc["Wavelength"] = analysis.get_wavelength()
    settings.loc["Field"] = analysis.get_field()
    settings.loc["Type"] = str(analysis.Settings.Type)
    settings.loc["UsePolarization"] = analysis.Settings.UsePolarization
    settings.loc["UseDashes"] = analysis.Settings.UseDashes

    # Get data and unpack
    data = []
    for ii in range(analysis.Results.NumberOfDataSeries):
        data.append(utils.zputils.unpack_dataseries(analysis.Results.DataSeries[ii]))

    if not len(data):
        data = pd.DataFrame()
    elif len(data) == 1:
        data = data[0]
    else:
        data = pd.concat(data, axis=1)

    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=settings,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
    )

    return analysis.complete(oncomplete, result)


def fft_through_focus_mtf_fromcfg(
    oss: OpticStudioSystem, cfgfile: str, oncomplete: OnComplete | str = OnComplete.Close
):
    """Wrapper around the OpticStudio FFT Through Focus MTF using a configuration file.

    For an in depth explanation of the parameters, see the Zemax OpticStudio user manual

    Parameters
    ----------
    oss : zospy.zpcore.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    cfgfile : str
        Full filepath (including extension) to a configuration file.
    oncomplete : OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.

    Returns
    -------
    AnalysisResult
        An FftThroughFocusMtf analysis result. Next to the standard data, the cfgfile will be added under 'UsedCfgFile'
    """
    analysis_type = constants.Analysis.AnalysisIDM.FftThroughFocusMtf

    # Create analysis
    analysis = new_analysis(oss, analysis_type)

    # Apply settings
    analysis.Settings.LoadFrom(cfgfile)

    # Calculate
    analysis.ApplyAndWaitForCompletion()

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Get settings
    settings = pd.Series(name="Settings", dtype=object)
    settings.drop(settings.index, inplace=True)

    settings.loc["SampleSize"] = str(analysis.Settings.SampleSize)
    settings.loc["DeltaFocus"] = analysis.Settings.DeltaFocus
    settings.loc["Frequency"] = analysis.Settings.Frequency
    settings.loc["NumberOfSteps"] = analysis.Settings.NumberOfSteps
    settings.loc["Wavelength"] = analysis.Settings.Wavelength.GetWavelengthNumber()  # Todo Evaluate
    settings.loc["Field"] = analysis.Settings.Field.GetFieldNumber()  # Todo Evaluate
    settings.loc["Type"] = str(analysis.Settings.Type)
    settings.loc["UsePolarization"] = analysis.Settings.UsePolarization
    settings.loc["UseDashes"] = analysis.Settings.UseDashes

    # Get data and unpack
    data = []
    for ii in range(analysis.Results.NumberOfDataSeries):
        data.append(utils.zputils.unpack_dataseries(analysis.Results.DataSeries[ii]))

    if not len(data):
        data = pd.DataFrame()
    elif len(data) == 1:
        data = data[0]
    else:
        data = pd.concat(data, axis=1)

    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=settings,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
        UsedCfgFile=cfgfile,
    )

    return analysis.complete(oncomplete, result)


def huygens_mtf(
    oss: OpticStudioSystem,
    pupil_sampling: str | int = "32x32",
    image_sampling: str | int = "32x32",
    image_delta: float = 0.0,
    wavelength: str | int = "All",
    field: str | int = "All",
    mtftype: constants.Analysis.Settings.Mtf.HuygensMtfTypes | str = "Modulation",
    maximum_frequency: float = 150.0,
    use_polarization: bool = False,
    use_dashes: bool = False,
    oncomplete: OnComplete | str = OnComplete.Close,
) -> AnalysisResult:
    """Wrapper around the OpticStudio Huygens MTF.

    For an in depth explanation of the parameters, see the Zemax OpticStudio user manual

    Parameters
    ----------
    oss : zospy.zpcore.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    pupil_sampling : str | int
        The pupil sampling, either string (e.g. '64x64') or int.
        The integer will be treated as a ZOSAPI Constants integer.
    image_sampling : str | int
        The image sampling, either string (e.g., '64x64') or int.
        The integer will be treated as a ZOSAPI Constants integer.
    image_delta : float
        The Image Delta, defaults to 0.0.
    wavelength : str | int
        The wavelength to use in the MTF. Either 'All' or an integer specifying the wavelength number.
    field : str | int
        The field to use in the MTF. Either 'All' or an integer specifying the field number.
    mtftype : zospy.constants.Analysis.Settings.Mtf.MtfTypes.Modulation
        The MTF type (e.g. `Modulation`) that is calculated.
    maximum_frequency : float
        The maximum frequency at which the MTF is calculated.
        Units are either cycles/mm or cycles/mr, depending on system setting.
        Defaults to 150.0, which is more appropriate when units are set to cycles/mm.
    use_polarization : bool
        Use polarization. Defaults to False.
    use_dashes : bool
        Use dashes. Defaults to False.
    oncomplete : OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.

    Returns
    -------
    AnalysisResult
        A HuygensMTF analysis result
    """
    analysis_type = constants.Analysis.AnalysisIDM.HuygensMtf

    # Create analysis
    analysis = new_analysis(oss, analysis_type)

    # Apply settings
    analysis.Settings.PupilSampleSize = getattr(
        constants.Analysis.SampleSizes, utils.zputils.standardize_sampling(pupil_sampling)
    )
    analysis.Settings.ImageSampleSize = getattr(
        constants.Analysis.SampleSizes, utils.zputils.standardize_sampling(image_sampling)
    )
    analysis.Settings.ImageDelta = image_delta
    analysis.set_wavelength(wavelength)
    analysis.set_field(field)
    analysis.Settings.Type = constants.process_constant(constants.Analysis.Settings.Mtf.HuygensMtfTypes, mtftype)
    analysis.Settings.UsePolarization = use_polarization
    analysis.Settings.UseDashes = use_dashes
    analysis.Settings.MaximumFrequency = maximum_frequency

    # Calculate
    analysis.ApplyAndWaitForCompletion()

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Get settings
    settings = pd.Series(name="Settings", dtype=object)

    settings.loc["PupilSampling"] = str(analysis.Settings.PupilSampleSize)
    settings.loc["ImageSampling"] = str(analysis.Settings.ImageSampleSize)
    settings.loc["ImageDelta"] = analysis.Settings.ImageDelta
    settings.loc["Wavelength"] = analysis.get_wavelength()
    settings.loc["Field"] = analysis.get_field()
    settings.loc["Type"] = str(analysis.Settings.Type)
    settings.loc["UsePolarization"] = analysis.Settings.UsePolarization
    settings.loc["UseDashes"] = analysis.Settings.UseDashes
    settings.loc["MaximumFrequency"] = analysis.Settings.MaximumFrequency

    # Get data and unpack
    data = []
    for ii in range(analysis.Results.NumberOfDataSeries):
        data.append(utils.zputils.unpack_dataseries(analysis.Results.DataSeries[ii]))

    if not len(data):
        data = pd.DataFrame()
    elif len(data) == 1:
        data = data[0]
    else:
        data = pd.concat(data, axis=1)

    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=settings,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
    )

    return analysis.complete(oncomplete, result)



"""Script to run all examples in a directory and save the output.

This script runs all examples using the `run_single_example.py` script and saves the output.
`uv` is used to run the examples in isolated environments, with only the required dependencies.
"""

from __future__ import annotations

import argparse
import sys
from pathlib import Path
from typing import Literal

from _common import process_example


def _handle_output_folder(output_folder: Path | None, example_folder: Path) -> Path:
    """Create the output folder if it does not exist and return the output path.

    Because only the output folder root is provided, the example name is appended to the output folder path.
    """
    if output_folder is not None:
        output_folder = output_folder.resolve(strict=False)
        if not output_folder.exists():
            print(f"Creating output directory at {output_folder}")
            output_folder.mkdir()

        return output_folder / example_folder.name

    return example_folder.resolve(strict=False)


def main(args: argparse.Namespace) -> Literal[1, 0]:
    directory: Path = args.example_directory

    if not directory.exists():
        print(f"Directory {directory} does not exist.")
        return 1

    if not directory.is_dir():
        print(f"{directory} is not a directory. Please provide a directory.")
        return 1

    subfolders = [f for f in directory.iterdir() if f.is_dir()]

    if len(subfolders) == 0:
        print(f"No subdirectories found in {directory}, exiting.")
        return 1

    success: dict[Path, bool] = {}

    for subfolder in subfolders:
        if len(list(subfolder.glob("*.ipynb"))) == 0:
            print(f"No notebooks found in {subfolder}, skipping.")
            continue

        output_folder = _handle_output_folder(args.output, subfolder)

        print(f"Processing example {subfolder}")
        zospy_location = args.zospy_location.resolve(strict=False) if args.zospy_location is not None else None
        success[subfolder] = process_example(
            subfolder,
            output_folder,
            editable_zospy_location=zospy_location,
            keep_extension_mode=args.keep_extension_mode,
        )

    print(
        "\n\n--- Results ---\n",
        *[f"{k}: {'Success' if v else 'Failure'}" for k, v in success.items()],
        sep="\n",
    )

    return 0 if all(success.values()) else 1


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run all examples in a directory and save the output.")
    (
        parser.add_argument(
            "example_directory",
            type=Path,
            help="Path to the directory containing the examples.",
        ),
    )
    parser.add_argument(
        "--zospy-location",
        type=Path,
        help="Path to the directory containing ZOSPy. If not provided, ZOSPy is installed from PyPI.",
        default=None,
    )
    parser.add_argument(
        "--output",
        type=Path,
        help="Path to save the output to. Defaults to the input example directory.",
        default=None,
    )
    parser.add_argument(
        "--keep-extension-mode",
        action="store_true",
        help="Do not change extension mode to standalone mode prior to running the notebook.",
    )

    args = parser.parse_args()

    sys.exit(main(args))
